/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/stat.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <sys/types.h>
#include <dirent.h>

#include <glib.h>
#include <gnome.h>

#include "tasks.h"
#include "server.h"
#include "protocol.h"
#include "guiprefs.h"
#include "guiutils.h"
#include "files.h"
#include "filelist.h"
#include "filetransfer.h"
#include "smalltrans.h"
#include "privs.h"
#include "pixmap.h"

#define FILES "Files"

/* ====================================== */

static void fileinfo_close(GtkButton *b,GtkWidget *top){
  gtk_widget_destroy(top);
}

static void fileinfo_delete(GtkButton *b,GdkEvent *ev,GtkWidget *top){
  gtk_widget_destroy(top);
}


static void create_fileinfo(FileEntry *fe, char *comment,
			    time_t mod_t, time_t cre_t){
			    
  GtkWidget *top,*table,*hbox,*vbox,*pixmap;
  GtkWidget *name_w,*t;
  GdkPixmap *pix,*mask;
  char buf[128],buf2[128];

  top=gnome_dialog_new(_("File Info"),GNOME_STOCK_BUTTON_CLOSE,NULL);
  vbox=GNOME_DIALOG(top)->vbox;

  hbox=gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,2);

  fileentry_get_icon(fe,&pix,&mask);
  pixmap=gtk_pixmap_new(pix,mask);
  gtk_box_pack_start(GTK_BOX(hbox),pixmap,FALSE,FALSE,2);
  name_w=gtk_entry_new();
  if(fe->name!=NULL)
    gtk_entry_set_text(GTK_ENTRY(name_w),fe->name);
  gtk_entry_set_editable(GTK_ENTRY(name_w),FALSE);
  gtk_box_pack_start(GTK_BOX(hbox),name_w,TRUE,TRUE,2);

  hbox=gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,2);
  table=gtk_table_new(2,5,FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(_("File type : ")),0,1,0,1);
  gtk_misc_set_alignment(GTK_MISC(t),1.0,0.0);
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(_("File creator : ")),0,1,1,2);
  gtk_misc_set_alignment(GTK_MISC(t),1.0,0.0);
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(_("Date modified : ")),0,1,2,3);
  gtk_misc_set_alignment(GTK_MISC(t),1.0,0.0);
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(_("Date created : ")),0,1,3,4);
  gtk_misc_set_alignment(GTK_MISC(t),1.0,0.0);
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(_("Size : ")),0,1,4,5);
  gtk_misc_set_alignment(GTK_MISC(t),1.0,0.0);

  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new((fe->type==NULL || fe->is_folder)?"n/a":fe->type),1,2,0,1);
  gtk_misc_set_alignment(GTK_MISC(t),0.0,0.0);
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new((fe->creator==NULL || fe->is_folder)?"n/a":fe->creator),1,2,1,2);
  gtk_misc_set_alignment(GTK_MISC(t),0.0,0.0);
  strcpy(buf,ctime(&mod_t));buf[strlen(buf)-1]=0;
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(buf),1,2,2,3);
  gtk_misc_set_alignment(GTK_MISC(t),0.0,0.0);
  strcpy(buf,ctime(&cre_t));buf[strlen(buf)-1]=0;
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(buf),1,2,3,4);
  gtk_misc_set_alignment(GTK_MISC(t),0.0,0.0);
  if(fe->is_folder){
    strcpy(buf,"n/a");
  } else {
    if(fe->size>1024){
      if(fe->size>(1024*1024)){
	sprintf(buf2,"%d MB",fe->size>>20);
      } else {
	sprintf(buf2,"%d KB",fe->size>>10);
      }
      sprintf(buf,"%s (%d bytes)",buf2,fe->size);
    } else {
      sprintf(buf,"%d bytes",fe->size);
    }
  }
  gtk_table_attach_defaults(GTK_TABLE(table),t=gtk_label_new(buf),1,2,4,5);
  gtk_misc_set_alignment(GTK_MISC(t),0.0,0.0);
  gtk_box_pack_start(GTK_BOX(hbox),table,FALSE,FALSE,2);

  pixmap=gtk_label_new(_("Comment:"));
  gtk_misc_set_alignment(GTK_MISC(pixmap),0.0,0.0);
  gtk_box_pack_start(GTK_BOX(vbox),pixmap,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),t=gtk_text_new(NULL,NULL),TRUE,TRUE,0);
  if(comment!=NULL)
    gtk_text_insert(GTK_TEXT(t),NULL,NULL,NULL,comment,strlen(comment));
  gnome_dialog_button_connect(GNOME_DIALOG(top),0,
			      GTK_SIGNAL_FUNC(fileinfo_close),
			      (gpointer)top);
  gtk_signal_connect(GTK_OBJECT(top),"delete-event",
		     GTK_SIGNAL_FUNC(fileinfo_delete),
		     (gpointer)top);
  gtk_widget_show_all(top);
}

/* ====================================== */
static void localfiles_chdir_to_path(FileList *fl,int num,gpointer data);

void fileentry_free(FileEntry *fe){
  if(fe==NULL)
    return;
  if(fe->name!=NULL)
    free(fe->name);
  free(fe);
}

FileEntry *fileentry_new_from_object(HLObject *o){
  FileEntry *fe=(FileEntry *)malloc(sizeof(FileEntry));

  fe->name=strdup(o->data.filelistentry->name);
  memcpy(fe->type,o->data.filelistentry->f.type,4);
  memcpy(fe->creator,o->data.filelistentry->f.creator,4);
  fe->type[4]=0;fe->creator[4]=0;
  fe->is_folder=FALSE;
  fe->is_selected=FALSE;
  if(strncmp(fe->type,"fldr",4)==0){
    fe->is_folder=TRUE;
  }
  fe->size=o->data.filelistentry->f.size;
  return fe;
}

FileEntry *fileentry_new_from_stat(char *name, struct stat *buf){
  FileEntry *fe=(FileEntry *)malloc(sizeof(FileEntry));
  fe->name=strdup(name);
  strcpy(fe->type,"n/a");
  strcpy(fe->creator,"n/a");
  if(S_ISDIR(buf->st_mode)){
    fe->size=buf->st_nlink;
    fe->is_folder=TRUE;
  } else {
    fe->size=buf->st_size;
    fe->is_folder=FALSE;
  }
  fe->is_selected=FALSE;
  return fe;
}

void fileentry_get_icon(FileEntry *fe,
			GdkPixmap **icon,
			GdkPixmap **mask){
  char *name;
  if(fe->is_folder){
    char buf[1024];
    strcpy(buf,fe->name);
    g_strdown(buf);
    if(strstr(buf,"upload")!=NULL)
      name="folder_green";
    else 
      name="folder_blue";
  } else {
    if(strncmp(fe->type,"HTft",4)==0)
      name="file_red";
    else
      name="file";
  }
  pixmap_get(name,icon,mask);
} 

static gboolean files_cache_remove_item(gpointer key,
					gpointer value,
					gpointer data){
  free(key);
  objects_destroy((HLObject **)data);
  return TRUE;
}

static void files_clear_cache(gpointer dummy, FileData *f){
  if(f->remote_cache==NULL)
    return;
  g_hash_table_foreach_remove(f->remote_cache,files_cache_remove_item,NULL);
  g_hash_table_destroy(f->remote_cache);
  f->remote_cache=NULL;
}

char *files_get_path(FileList *fl){
  char *p,*t;
  if(fl->numpaths<=0)
    return strdup("/");
  p=g_strjoinv("/",fl->path);
  t=g_strconcat("/",p,NULL);
  free(p);
  return t;
}

static void localfiles_refresh(FileList *fl, gpointer data){
  FileData *f=(FileData *)data;
  int count=0,alloc=0;
  FileEntry **fe=NULL;
  DIR *dir;
  char *path=NULL,*t=NULL;
  struct stat buf;
  struct dirent *d;
  
  path=files_get_path(f->local_files);
  dir=opendir(path);
  if(dir==0){
    show_error_errno(f->c,NULL,path);
    free(path);
    free(t);
    count=f->local_files->numpaths-2;
    if(count<-1) count=-1;
    localfiles_chdir_to_path(f->local_files,count,f);
    return;
  }
  while((d=readdir(dir))!=NULL){
    if((hide_local_dotfiles==TRUE && d->d_name[0]=='.') ||
       strcmp(d->d_name,".")==0 || strcmp(d->d_name,"..")==0)
      continue;
    if(t!=NULL)
      free(t);
    t=g_strconcat(path,"/",d->d_name,NULL);
    if(stat(t,&buf)!=0){
      show_error_errno(f->c,NULL,t);
      continue;
    }
    if(count==alloc){
      if(alloc==0) alloc=10; else alloc*=2;
      fe=realloc(fe,sizeof(FileEntry *)*alloc);
    }
    fe[count++]=fileentry_new_from_stat(d->d_name,&buf);
  }
  if(t!=NULL)
    free(t);
  free(path);
  closedir(dir);
  filelist_clear(f->local_files);
  filelist_set_contents(f->local_files,count,fe);
}

static void localfiles_chdir_to(FileData *f,char *path){
  char **v;

  if(strcmp(path,"/")==0)
    v=NULL;
  else {
    while(path[0]=='/') path++;
    v=g_strsplit(path,"/",-1);
  }
  filelist_set_path(f->local_files,v);
  localfiles_refresh(NULL,f);
}

static void localfiles_chdir_to_path(FileList *fl,int num,gpointer data){
  FileData *f=(FileData *)data;
  char *path,*t;

  if(num==-1){
    localfiles_chdir_to(f,"/");
    return;
  }
  if(num==fl->numpaths-1)
    return;
  t=fl->path[num+1];
  fl->path[num+1]=NULL;
  path=files_get_path(fl);
  fl->path[num+1]=t;
  localfiles_chdir_to(f,path);
  free(path);
}

static void localfiles_change_selection(FileList *fl,gpointer data){
  FileData *fd=(FileData *)data;
  gutils_nbpage_toolbar_set_sensitive(fd->notebook_page,2,
				      (fl->count_selected>0 &&
				       privs_check(fd->c,PRIV_UPLOAD_FILE))
				      ?TRUE:FALSE);
}
static void localfiles_select_one(FileList *fl,FileEntry *which,gpointer data){
  FileData *fd=(FileData *)data;
  if(which->is_folder==TRUE){
    char *p=files_get_path(fl);
    char *t=g_strconcat(p,"/",which->name,NULL);
    free(p);
    localfiles_chdir_to(fd,t);
    free(t);
  } else {
    filetransfer_start(fd,TRUE,which);
  }
}

static void localfiles_mkdir(FileList *fl,char *name, gpointer data){
  char *p,*t;
  if(name==NULL || strlen(name)==0) return;
  p=files_get_path(fl);
  t=g_strconcat(p,"/",name,NULL);
  free(p);
  if(mkdir(t,0777)==-1){
    char buf[2048];
    sprintf(buf,_("Could not create directory\n%s:\n"),p);
    show_error_errno(((FileData *)data)->c,NULL,buf);
  }
  free(t);  
}

static void localfiles_delete(FileList *fl,int which, gpointer data){
  char *p,*t;
  if(which==-1) return;
  p=files_get_path(fl);
  t=g_strconcat(p,"/",fl->files[which]->name,NULL);
  free(p);
  if(remove(t)==-1){
    char buf[2048];
    sprintf(buf,_("Unable to remove\n%s:\n"),p);
    show_error_errno(((FileData *)data)->c,NULL,buf);
  }
  free(t);  
}

static void localfiles_stat(FileList *fl,FileEntry *which, gpointer data){
  char *p,*t;
  char buf[2048];
  struct stat st;

  if(which==NULL) return;
  p=files_get_path(fl);
  t=g_strconcat(p,"/",which->name,NULL);
  free(p);
  if(lstat(t,&st)==-1){
    sprintf(buf,_("Unable to stat\n%s:\n"),p);
    show_error_errno(((FileData *)data)->c,NULL,buf);
  } else {
    create_fileinfo(which,NULL,st.st_mtime,st.st_ctime);
  }
  free(t);  
}

static void local_start_upload(FileList *fl,
			       FileEntry *which,
			       gpointer data){
  FileData *fd=(FileData *)data;

  if(which->is_folder==FALSE)
    filetransfer_start(fd,TRUE,which);
}

static char **headandappend_path(char **path,int count,char *tail){
  char **p;
  int i;

  if(count<0) return NULL;
  p=malloc(sizeof(char *)*(count+((tail==NULL)?1:2)));
  for(i=0;i<count;i++)
    p[i]=strdup(path[i]);
  if(tail!=NULL)
    p[count++]=strdup(tail);
  p[count]=NULL;
  return p;
}

static void remote_change_selection(FileList *fl,gpointer data){
  FileData *fd=(FileData *)data;
  gutils_nbpage_toolbar_set_sensitive(fd->notebook_page,3,
				      (fl->count_selected>0 &&
				       privs_check(fd->c,PRIV_DOWNLOAD_FILE))
				       ?TRUE:FALSE);
}

static void remote_filelist_fill_from_objects(FileData *fd,HLObject **ob){
  FileEntry **fe=NULL;
  int i,count;

  if(ob==NULL) return;
  for(count=0;ob[count]!=NULL;count++);
  if(count==0) return;
  fe=malloc(sizeof(FileData *)*count);
  for(i=0;i<count;i++)
    fe[i]=fileentry_new_from_object(ob[i]);
  filelist_set_contents(fd->remote_files,count,fe);
}

static gboolean remote_filelist_fill_list(gpointer data){
  HLTransaction *r=(HLTransaction *)data;
  FileData *f=(FileData *)r->data;
  HLObject **o,*err;
  int count;
  char *p;

  if(r->w.error!=0){
    err=transaction_find_object(r,HLO_ERRORMSG);
    if(err!=NULL)
      show_error(f->c,NULL,g_strconcat(_("Can't list directory:\n"),
			     err->data.string,NULL));
  } else {
    o=transaction_extract_objects(r,HLO_FILEENTRY,&count);
    remote_filelist_fill_from_objects(f,o);
    p=files_get_path(f->remote_files);
    g_hash_table_insert(f->remote_cache,p,o);
  }
  transaction_destroy(r);
  if(f->task!=NULL)
    task_remove(f->task);
  f->task=NULL;
  return FALSE;
}

static void remote_filelist_get_reply(Connection *c,
				      HLTransaction *t,
				      HLTransaction *r,
				      gpointer data){
  FileData *f=(FileData *)data;
  transaction_destroy(t);
  transaction_set_task(r,f->task,0,1.0);
  transaction_read_objects(c,r);
  r->data=f;
  gtk_idle_add(remote_filelist_fill_list,r);
}

static void remote_send_query(Connection *c,gpointer data){
  HLTransaction *t=transaction_new(HLCT_FOLDERLIST,NULL,FALSE);
  FileData *f=(FileData *)data;
  t->data=f;
  if(f->remote_files->numpaths>0){
    transaction_add_object(t,path_new(f->remote_files->path,
				      f->remote_files->numpaths));
  }
  server_transaction_reply_set(c,t,remote_filelist_get_reply,f);
  transaction_send(t,c);
}

static void remote_remove_task(Task *t){
  FileData *f=(FileData *)t->user_data;
  if(f==NULL) return;
  f->task=NULL;
}

static void remote_refresh(FileList *fl,gpointer data){
  FileData *f=(FileData *)data;
  char *p;
  HLObject **ob;

  if(f->task!=NULL)
    return;
  filelist_clear(f->remote_files);
  p=files_get_path(fl);
  if((ob=g_hash_table_lookup(f->remote_cache,p))!=NULL){
    remote_filelist_fill_from_objects(f,ob);
    free(p);
    return;
  }
  f->task=task_new(f->c->bookmark->host);
  f->task->user_data=f;
  f->task->user_cancel=remote_remove_task;
  sprintf(f->task->text,_("Retrieving %s ..."),p);
  free(p);
  task_update(f->task,TRUE,0.0);
  task_add(f->task);
  server_transaction_start(f->c,remote_send_query,f);
}

static void remote_real_refresh(FileList *fl,gpointer data){
  FileData *f=(FileData *)data;
  char *p,*orig_p;
  HLObject **ob;

  if(f->task!=NULL)
    return;
  p=files_get_path(fl);
  if(g_hash_table_lookup_extended(f->remote_cache,p,
				  (gpointer *)&orig_p,(gpointer *)&ob)==TRUE){
    g_hash_table_remove(f->remote_cache,p);
    free(orig_p);
    objects_destroy(ob);
  }
  free(p);
  remote_refresh(fl,data);
}

static void remote_chdir_to_path(FileList *fl,int num,gpointer data){
  FileData *f=(FileData *)data;
  char **p;
  if(f->task!=NULL || num==fl->numpaths)
    return;
  if(num==-1){
    p=NULL;
  } else {
    p=headandappend_path(fl->path,num+1,NULL);
  }
  filelist_set_path(fl,p);
  remote_refresh(fl,f);
}

static void remote_simple_send(Connection *c,gpointer data){
   HLTransaction *t=(HLTransaction *)data;
   transaction_send(t,c);
}

static void remote_mkdir(FileList *fl,char *name, gpointer data){
  HLTransaction *t;
  FileData *f=(FileData *)data;

  if(name==NULL || strlen(name)==0) return;
  t=transaction_new(HLCT_CREATEFOLDER,NULL,FALSE);
  transaction_add_object(t,create_string(HLO_FILENAME,name));
  if(fl->numpaths>0){
    transaction_add_object(t,path_new(fl->path,
				      fl->numpaths));
  }
  server_transaction_reply_set(f->c,t,server_reply_default,
			       strdup(_("Could not make directory:\n")));
  server_transaction_start(f->c,remote_simple_send,(gpointer)t);
}

static void remote_delete(FileList *fl,int which, gpointer data){
  FileData *f=(FileData *)data;
  HLTransaction *t;

  if(which==-1) return;
  t=transaction_new(HLCT_MOVETOTRASH,NULL,FALSE);
  transaction_add_object(t,create_string(HLO_FILENAME,fl->files[which]->name));
  if(fl->numpaths>0){
    transaction_add_object(t,path_new(fl->path,
				      fl->numpaths));
  }
  server_transaction_reply_set(f->c,t,server_reply_default,
			       strdup(_("Could not delete:\n")));
  server_transaction_start(f->c,remote_simple_send,(gpointer)t);
}

static gboolean remote_got_stat(gpointer data){
  HLTransaction *t=data;
  HLObject *filename,*comment=NULL,*size,*type,*creator,*created,*modified;
  FileEntry fe;
  time_t mod_t,cre_t;

  task_remove(t->task);
  filename=transaction_find_object(t,HLO_FILENAME);
  comment=transaction_find_object(t,HLO_COMMENT);
  size=transaction_find_object(t,HLO_INFOSIZE);
  created=transaction_find_object(t,HLO_INFOCREATED);
  modified=transaction_find_object(t,HLO_INFOMODIFIED);
  creator=transaction_find_object(t,HLO_INFOCREATOR);
  type=transaction_find_object(t,HLO_INFOTYPE);

  fe.name=filename->data.string;
  if(size!=NULL)
    fe.size=size->data.number;
  else 
    fe.size=0;
  strncpy(fe.type,type->data.string,4);
  strncpy(fe.creator,creator->data.string,4);
  fe.type[4]=0;fe.creator[5]=0;
  mod_t=date_to_unix(modified->data.datetime);
  cre_t=date_to_unix(created->data.datetime);
  fe.is_folder=(strncmp(fe.type,"fldr",4)==0)?TRUE:FALSE;
  create_fileinfo(&fe,comment==NULL?NULL:comment->data.string,mod_t,cre_t);
  transaction_destroy(t);
  return FALSE;
}

static void remote_stat_get_reply(Connection *c,
				  HLTransaction *t,
				  HLTransaction *r,
				  gpointer data){
  FileData *f=(FileData *)t->data;
  transaction_set_task(r,t->task,0.5,1.0);
  transaction_destroy(t);
  transaction_read_objects(c,r);
  r->data=f;
  gtk_idle_add(remote_got_stat,r);
}
static void remote_stat(FileList *fl,FileEntry *which, gpointer data){
  FileData *f=(FileData *)data;
  HLTransaction *t;
  Task *task;

  if(which==NULL) return;
  t=transaction_new(HLCT_GETFILEINFO,NULL,FALSE);
  transaction_add_object(t,create_string(HLO_FILENAME,which->name));
  if(fl->numpaths>0){
    transaction_add_object(t,path_new(fl->path,
				      fl->numpaths));
  }
  t->data=f;
  task=task_new(f->c->bookmark->host);
  transaction_set_task(t,task,0.0,0.5);
  sprintf(task->text,_("Retrieving info for %s..."),which->name);
  task_add(task);
  task_update(task,TRUE,0.0);
  server_transaction_reply_set(f->c,t,remote_stat_get_reply,f);
  server_transaction_start(f->c,remote_simple_send,t);
}

static void remote_select_one(FileList *fl,FileEntry *which,gpointer data){
  FileData *fd=(FileData *)data;
  if(which->is_folder==TRUE){
    char **p=headandappend_path(fl->path,fl->numpaths,which->name);
    filelist_set_path(fl,p);
    remote_refresh(fl,fd);
  } else {
    filetransfer_start(fd,FALSE,which);
  }
}

static void remote_start_download(FileList *fl,
				  FileEntry *which,
				  gpointer data){
  FileData *fd=(FileData *)data;

  if(which->is_folder==FALSE)
    filetransfer_start(fd,FALSE,which);
}

static void files_destroy_gui(gpointer dummy, FileData *f){
  if(f->remote_files!=NULL)
    filelist_destroy(f->remote_files);
  if(f->local_files!=NULL)
    filelist_destroy(f->local_files);
  if(f->notebook_page!=NULL)
    gutils_nbpage_destroy(f->notebook_page);
  f->notebook_page=NULL;
  f->local_files=NULL;
  f->remote_files=NULL;
}

static void files_destroy(Connection *c,gpointer data){
  FileData *f=(FileData *)data;
  hooks_set_data(c,FILES,NULL);

  if(f->task!=NULL){
    task_remove(f->task);
    f->task=NULL;
  }
  files_destroy_gui(NULL,f);
  files_clear_cache(NULL,f);
  free(f);
}

static void files_clear(GtkButton *b, gpointer data){
  FileData *f=(FileData *)data;
  filelist_deselect_all(NULL,f->remote_files);
  filelist_deselect_all(NULL,f->local_files);
}

/* Added by Caspian */
static void files_select_all(GtkButton *b, gpointer data){
  FileData *f=(FileData *)data;
  filelist_select_all(f->remote_files);
  filelist_select_all(f->local_files);
}

static void files_upload(GtkButton *b, gpointer data){
  FileData *f=(FileData *)data;
  filelist_start_transfer(f->local_files);
}

static void files_download(GtkButton *b, gpointer data){
  FileData *f=(FileData *)data;
  filelist_start_transfer(f->remote_files);
}

static GnomeUIInfo files_toolbar[]={
  GNOMEUIINFO_ITEM_STOCK("Close",N_("Close userlist"),
			 files_destroy_gui,
			 HL_STOCK_PIXMAP_CLOSE_PAGE),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK(N_("Upload"),N_("Upload file(s)"),
			 files_upload,
			 HL_STOCK_PIXMAP_UPLOAD),
  GNOMEUIINFO_ITEM_STOCK(N_("Download"),N_("Upload file(s)"),
			 files_download,
			 HL_STOCK_PIXMAP_DOWNLOAD),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK("All",N_("Select All"),
			 files_select_all,
                         GNOME_STOCK_PIXMAP_JUMP_TO),
  GNOMEUIINFO_ITEM_STOCK("None",N_("Clear Selections"),
			 files_clear,
			 GNOME_STOCK_PIXMAP_CLEAR),
  GNOMEUIINFO_END
};

static FLOps remote_ops={
  TRUE,
  remote_change_selection,
  remote_real_refresh,
  remote_mkdir,
  remote_start_download,
  remote_delete,
  remote_chdir_to_path,
  remote_select_one,
  remote_stat
};

static FLOps local_ops={
  FALSE,
  localfiles_change_selection,
  localfiles_refresh,
  localfiles_mkdir,
  local_start_upload,
  localfiles_delete,
  localfiles_chdir_to_path,
  localfiles_select_one,
  localfiles_stat
};


static void files_create_gui(FileData *f){
  GtkWidget *paned;

  if(f->notebook_page!=NULL)
    return;
  f->notebook_page=gutils_nbpage_new("Main",files_toolbar,f->c,_("Files"),"files",f);
  paned=gtk_hpaned_new();
  f->remote_files=filelist_new(f->c,&remote_ops,f);
  gtk_paned_add1(GTK_PANED(paned),f->remote_files->frame);

  f->local_files=filelist_new(f->c,&local_ops,f);
  gtk_paned_add2(GTK_PANED(paned),f->local_files->frame);

  guiprefs_add_paned(GTK_PANED(paned),"Main/Files/PanePos");
  gutils_nbpage_set_main(f->notebook_page,paned);
  localfiles_chdir_to(f,default_local_dir);
  remote_chdir_to_path(f->remote_files,-1,f);
}

void check_files(Connection *c){
  FileData *f;

  if((f=(FileData *)hooks_get_data(c,FILES))==NULL){
    hooks_create(c,FILES,NULL,files_destroy);
    f=(FileData *)malloc(sizeof(FileData));
    hooks_set_data(c,FILES,(gpointer)f);
    f->c=c;
    f->local_files=NULL;
    f->remote_cache=g_hash_table_new(g_str_hash,g_str_equal);
    f->notebook_page=NULL;
    f->remote_files=NULL;
    f->task=NULL;
  }
  if(auto_open_files==TRUE)
    files_create_gui(f);
}

void show_files(Connection *c){
  FileData *f=(FileData *)hooks_get_data(c,FILES);
  if(f==NULL)
    check_files(c);
  f=(FileData *)hooks_get_data(c,FILES);
  if(f->notebook_page==NULL)
    files_create_gui(f);
  gutils_nbpage_to_front(f->notebook_page);
}
