/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnome.h>

#include "guiprefs.h"
#include "main.h"
#include "hldat.h"
#include "pixmap.h"
#include "hldat.h"

typedef struct {
  char *config_path;
  GtkWidget *pane;
} PanePrefs;

typedef struct {
  char *config_path;
  GtkCList *list;
  int num_columns;
  int *column_width;
} CListPrefs;

static void handle_window_resize(GtkWidget *widget,
				 GtkAllocation *allocation,
				 gpointer path){
  char *p=g_strconcat("/",APPLICATION_NAME,"/",(char *)path,NULL);
  char buf[512];
  
  sprintf(buf,"%d,%d",allocation->width,allocation->height);
  gnome_config_set_string(p,buf);
  g_free(p);
  gnome_config_sync();
}

void guiprefs_add_window(GtkWindow *w,const char *path){
  char *p=g_strconcat("/",APPLICATION_NAME,"/",path,NULL);
  char *s=gnome_config_get_string(p),**v=NULL;

  if(s!=NULL){
    v=g_strsplit(s,",",2);
    gtk_window_set_default_size(w,atoi(v[0]),atoi(v[1]));
    g_strfreev(v);
    g_free(s);
  }
  g_free(p);
  gtk_signal_connect(GTK_OBJECT(w),"size-allocate",
		     GTK_SIGNAL_FUNC(handle_window_resize),
		     (gpointer)path);
}

static void handle_paned_resize(GtkWidget *widget,
				GtkAllocation *allocation,
				gpointer data){
  PanePrefs *pp=(PanePrefs *)data;
  int pos;
  
  if(GTK_IS_VPANED(pp->pane)){
    pos=allocation->height;
  } else {
    pos=allocation->width;
  }
  gnome_config_set_int(pp->config_path,pos);
  gnome_config_sync();
}

void guiprefs_add_paned(GtkPaned *w,const char *path){
  PanePrefs *pp=g_malloc(sizeof(PanePrefs));
  char *p=g_strconcat("/",APPLICATION_NAME,"/",path,NULL);
  int pos=gnome_config_get_int(p);
  GList *l=gtk_container_children(GTK_CONTAINER(w));
  GtkWidget *h1=(GtkWidget *)l->data;;

  pp->config_path=p;
  if(pos!=0){
    gtk_paned_set_position(GTK_PANED(w),pos);
  }
  pp->pane=GTK_WIDGET(w);
  gtk_signal_connect(GTK_OBJECT(h1),"size-allocate",
		     GTK_SIGNAL_FUNC(handle_paned_resize),
		     (gpointer)pp);
  g_list_free(l);
}

static void handle_clist_resize_column(GtkCList *clist,
				       gint column,
				       gint width,
				       gpointer user_data){
  CListPrefs *cp=(CListPrefs*)user_data;
  char buf[512],num[64];
  int i;

  cp->column_width[column]=width;
  buf[0]=0;
  for(i=0;i<cp->num_columns;i++){
    sprintf(num,"%d",cp->column_width[i]);
    strcat(buf,num);
    if(i+1<cp->num_columns)
      strcat(buf,",");
  }
  gnome_config_set_string(cp->config_path,buf);
  gnome_config_sync();
}

void guiprefs_add_clist(GtkCList *w,const char *path,const char *def_widths){
  CListPrefs *cp=g_malloc(sizeof(CListPrefs));
  char *p=g_strconcat("/",APPLICATION_NAME,"/",path,NULL);
  char *s=gnome_config_get_string(p),**v=NULL;
  GtkArg arg;
  int i;

  cp->list=w;
  cp->config_path=p;
  arg.name="n_columns";
  gtk_object_getv(GTK_OBJECT(w),1,&arg);
  cp->num_columns=GTK_VALUE_INT(arg);
  cp->column_width=g_malloc(sizeof(*cp->column_width)*cp->num_columns);
  if(s==NULL){
    s=g_strdup(def_widths);
  }
  v=g_strsplit(s,",",cp->num_columns);
  for(i=0;i<cp->num_columns;i++){
    if((cp->column_width[i]=atoi(v[i]))>0)
      gtk_clist_set_column_width(w,i,cp->column_width[i]);
  }
  g_strfreev(v);
  g_free(s);
  gtk_signal_connect(GTK_OBJECT(w),"resize-column",
		     GTK_SIGNAL_FUNC(handle_clist_resize_column),
		     (gpointer)cp);
}

/*========================================*/
/*========================================*/
/*========================================*/
/*========================================*/

gboolean prefs_dblclick_connect_query_bookmarks=TRUE;
gboolean prefs_dblclick_connect_query_trackers=TRUE;

gboolean tracker_sort_only_by_ip=TRUE;

int prefs_task_refresh_msec=100;

char *nickname=NULL;
gboolean nickname_append_max_speed=FALSE;
int user_icon_number=160;
int nickname_append_speed_seconds=60;
gboolean passwords_are_hidden=TRUE;
int notebook_tab_side=GTK_POS_LEFT;
gboolean notebook_has_icons=TRUE;
GdkColor notebook_tab_highlight_color;

gboolean dont_show_agreements=FALSE;
PrefsFont agreement_font=DEF_PREF_FONT;

gboolean auto_open_news=TRUE;
gboolean news_big_messagewindow=TRUE;
PrefsFont news_font=DEF_PREF_FONT;

gboolean auto_open_pubchat=FALSE;
gboolean auto_popup_pubchat=FALSE;

gboolean auto_open_files=TRUE;
gboolean mix_files_and_folders=FALSE;
gboolean hide_local_dotfiles=TRUE;
gboolean cache_remote_directories=TRUE;
char *default_local_dir=NULL;
gboolean strip_spaces_on_local_files=TRUE;
gboolean sanitize_local_filenames=FALSE;
gboolean download_convert_text=TRUE;
gboolean upload_convert_text=TRUE;
gboolean download_remove_null_files=TRUE;
gboolean download_queue=TRUE;
gboolean upload_queue=TRUE;
gboolean files_small_sizes=TRUE;
int retry_count,retry_delay;

gboolean prefs_messages_show_with_reply=FALSE;
gboolean prefs_messages_reply_quoted=FALSE;
PrefsFont chat_font=DEF_PREF_FONT;

gboolean auto_open_userlist=TRUE;
gboolean userlist_aa_canvas=TRUE;
gboolean userlist_sort_name=TRUE;
gboolean userlist_group_by_status=TRUE;
gboolean userlist_use_icons=TRUE;
gboolean userlist_use_status_icons=TRUE;
GdkColor userlist_colors[4],userlist_highlight_color;
gushort userlist_highlight_color_alpha=0x8000;
PrefsFont userlist_font=DEF_PREF_FONT;
gboolean userlist_dock_right=FALSE;

GList *hotline_data_files=NULL;
gboolean use_hotline_icons=TRUE;

static gboolean initialized_colors=FALSE;
static GtkWidget *property_window=NULL;
static GtkWidget *nickname_entry_widget=NULL;
static GtkWidget *default_local_dir_widget=NULL;
static GtkWidget *icon_pixmap_widget=NULL;
static GtkWidget *icon_files_widget=NULL;

#ifdef HAVE_ESD
static gboolean play_sound_types[HL_SOUND_MAX-HL_SOUND_CHAT];
static gboolean play_sounds=FALSE;
#endif

static void read_string(ConfigElement *e){
  if(*((char **)e->addr)!=NULL)
    free(*(char **)e->addr);
  *(char **)e->addr=gnome_config_get_string(e->path);
}
static void write_string(ConfigElement *e){
  if(*((char **)e->addr)==NULL)
    return;
  gnome_config_set_string(e->path,*((char **)e->addr));
}
static void read_number(ConfigElement *e){
  *(int *)e->addr=gnome_config_get_int(e->path);
}
static void write_number(ConfigElement *e){
  gnome_config_set_int(e->path,*(int *)e->addr);
}
static void read_bool(ConfigElement *e){
  *(gboolean *)e->addr=(gnome_config_get_int(e->path)==0)?FALSE:TRUE;
}
static void write_bool(ConfigElement *e){
  gnome_config_set_int(e->path,(*(gboolean *)e->addr)==FALSE?0:1);
}
static void read_color(ConfigElement *e){
  int red,green,blue;
  char *s;
  GdkColor *c=(GdkColor *)e->addr;
  s=gnome_config_get_string(e->path);
  sscanf(s,"%d %d %d",&red,&green,&blue);
  g_free(s);
  c->red=red<<8;
  c->green=green<<8;
  c->blue=blue<<8;
}
static void write_color(ConfigElement *e){
  char buf[256];
  sprintf(buf,"%d %d %d",((GdkColor *)e->addr)->red>>8,
	  ((GdkColor *)e->addr)->green>>8,((GdkColor *)e->addr)->blue>>8);
  gnome_config_set_string(e->path,buf);
}
static void read_stringlist(ConfigElement *e){
  GList **gl=e->addr;
  char *s,**v;
  int i;
  while(*gl!=NULL){
    free((*gl)->data);
    *gl=g_list_remove(*gl,(*gl)->data);
  }
  s=gnome_config_get_string(e->path);
  if(s==NULL) return;
  v=g_strsplit(s,":",0);
  for(i=0;v[i]!=NULL;i++)
    *gl=g_list_append(*gl,strdup(v[i]));
  g_strfreev(v);
}
static void write_stringlist(ConfigElement *e){
  GList **gl=e->addr,*l;
  char *s,**v;
  int i,count;

  for(l=*gl,count=0;l!=NULL;l=l->next,count++);
  v=calloc(sizeof(char *),count+1);
  v[count]=NULL;
  for(l=*gl,i=0;l!=NULL;l=l->next,i++)
    v[i]=l->data;
  s=g_strjoinv(":",v);
  gnome_config_set_string(e->path,s);
}
static void read_font(ConfigElement *e){
  PrefsFont *pf=(PrefsFont *)(e->addr);
  char buf[256];
  if(pf->font_name!=NULL)
    free(pf->font_name),pf->font_name=NULL;
  if(pf->font!=NULL)
    gdk_font_unref(pf->font),pf->font=NULL;
  sprintf(buf,"%s_use",e->path);
  pf->use_custom_font=gnome_config_get_int(buf)!=0?TRUE:FALSE;
  if(pf->use_custom_font==TRUE){
    sprintf(buf,"%s_font=fixed",e->path);
    pf->font_name=gnome_config_get_string(buf);
    pf->font=gdk_font_load(pf->font_name);
    if(pf->font!=NULL)
      gdk_font_ref(pf->font);
  }
}
static void write_font(ConfigElement *e){
  PrefsFont *pf=(PrefsFont *)(e->addr);
  char buf[256];
  sprintf(buf,"%s_use",e->path);
  gnome_config_set_int(buf,pf->use_custom_font);
  if(pf->font_name!=NULL){
    sprintf(buf,"%s_font",e->path);
    gnome_config_set_string(buf,pf->font_name);
  }
}


static ConfigClass conftypes[]={
  {read_string,write_string},
  {read_number,write_number},
  {read_bool,write_bool},
  {read_color,write_color},
  {read_font,write_font},
  {read_stringlist,write_stringlist}
};
#define STRING_TYPE (&conftypes[0])
#define NUMBER_TYPE (&conftypes[1])
#define BOOL_TYPE (&conftypes[2])
#define COLOR_TYPE (&conftypes[3])
#define FONT_TYPE (&conftypes[4])
#define STRLIST_TYPE (&conftypes[5])

static ConfigElement confitems[]={
  {STRING_TYPE,"nickname=Lazy User",&nickname},
  {BOOL_TYPE,"doubleclick_queries_bookmarks=0",
   &prefs_dblclick_connect_query_bookmarks},
  {BOOL_TYPE,"doubleclick_queries_trackers=1",
   &prefs_dblclick_connect_query_trackers},
  {BOOL_TYPE,"tracker_sort_only_by_ip=0",&tracker_sort_only_by_ip},
  {NUMBER_TYPE,"icon_number=160",&user_icon_number},
  {BOOL_TYPE,"nickname_append_max_speed=0",&nickname_append_max_speed},
  {NUMBER_TYPE,"nickname_append_speed_seconds=60",
   &nickname_append_speed_seconds},
  {BOOL_TYPE,"dont_show_agreements=0",&dont_show_agreements},
  {BOOL_TYPE,"passwords_are_hidden=1",&passwords_are_hidden},
  {NUMBER_TYPE,"task_refresh_delay=500",&prefs_task_refresh_msec},
  {BOOL_TYPE,"auto_open_news=1",&auto_open_news},
  {BOOL_TYPE,"news_big_messagewindow=1",&news_big_messagewindow},
  {BOOL_TYPE,"auto_open_userlist=1",&auto_open_userlist},
  {BOOL_TYPE,"userlist_aa_canvas=1",&userlist_aa_canvas},
  {BOOL_TYPE,"userlist_sort_name=1",&userlist_sort_name},
  {BOOL_TYPE,"userlist_group_by_status=1",&userlist_group_by_status},
  {BOOL_TYPE,"userlist_use_icons=1",&userlist_use_icons},
  {BOOL_TYPE,"userlist_use_status_icons=1",&userlist_use_status_icons},
  {STRLIST_TYPE,"hotline_data_files=./hotline.dat",&hotline_data_files},
  {BOOL_TYPE,"use_hotline_icons=1",&use_hotline_icons},
  {COLOR_TYPE,"userlist_admin_ghost_color=128 0 0",
   &userlist_colors[ADMIN_GHOST_COLOR]},
  {COLOR_TYPE,"userlist_ghost_color=128 128 128",
   &userlist_colors[GHOST_COLOR]},
  {COLOR_TYPE,"userlist_admin_color=255 0 0",&userlist_colors[ADMIN_COLOR]},
  {COLOR_TYPE,"userlist_normal_color=0 0 0",&userlist_colors[NORMAL_COLOR]},
  {COLOR_TYPE,"userlist_highlight_color=32 32 127",&userlist_highlight_color},
  {NUMBER_TYPE,"userlist_highlight_color_alpha=32768",
   &userlist_highlight_color_alpha},
  {BOOL_TYPE,"auto_open_pubchat=1",&auto_open_pubchat},
  {BOOL_TYPE,"auto_popup_pubchat=0",&auto_popup_pubchat},
  {BOOL_TYPE,"prefs_messages_show_with_reply=0",
   &prefs_messages_show_with_reply},
  {BOOL_TYPE,"prefs_messages_reply_quoted=0",&prefs_messages_reply_quoted},
  {BOOL_TYPE,"auto_open_files=1",&auto_open_files},
  {BOOL_TYPE,"mix_files_and_folders=0",&mix_files_and_folders},
  {BOOL_TYPE,"hide_local_dotfiles=1",&hide_local_dotfiles},
  {BOOL_TYPE,"cache_remote_directories=1",&cache_remote_directories},
  {STRING_TYPE,"default_local_dir=/tmp",&default_local_dir},
  {BOOL_TYPE,"strip_spaces_on_local_files=1",&strip_spaces_on_local_files},
  {BOOL_TYPE,"sanitize_local_filenames=0",&sanitize_local_filenames},
  {BOOL_TYPE,"download_convert_text=1",&download_convert_text},
  {BOOL_TYPE,"upload_convert_text=1",&upload_convert_text},
  {BOOL_TYPE,"download_remove_null_files=1",&download_remove_null_files},
  {BOOL_TYPE,"download_queue=1",&download_queue},
  {BOOL_TYPE,"upload_queue=1",&upload_queue},
  {BOOL_TYPE,"files_small_sizes=1",&files_small_sizes},
  DEF_FONT_CONFIG("userlist_font",userlist_font),
  DEF_FONT_CONFIG("agreement_font",agreement_font),
  DEF_FONT_CONFIG("news_font",news_font),
  DEF_FONT_CONFIG("chat_font",chat_font),
  {NUMBER_TYPE,"notebook_tab_side=0",&notebook_tab_side},
  {BOOL_TYPE,"notebook_has_icons=1",&notebook_has_icons},
  {BOOL_TYPE,"userlist_dock_right=0",&userlist_dock_right},
  {COLOR_TYPE,"notebook_tab_highlight_color=127 0 0",&notebook_tab_highlight_color},
  {NUMBER_TYPE,"retry_count=10",&retry_count},
  {NUMBER_TYPE,"retry_delay=10",&retry_delay},

#ifdef HAVE_ESD
  {BOOL_TYPE,"play_sounds=1",&play_sounds},
  {BOOL_TYPE,"play_sound_chat=1",&play_sound_types[0]},
  {BOOL_TYPE,"play_sound_logged_in=1",&play_sound_types[1]},
  {BOOL_TYPE,"play_sound_logout=1",&play_sound_types[2]},
  {BOOL_TYPE,"play_sound_broadcast=1",&play_sound_types[3]},
  {BOOL_TYPE,"play_sound_login=1",&play_sound_types[4]},
  {BOOL_TYPE,"play_sound_error=1",&play_sound_types[5]},
  {BOOL_TYPE,"play_sound_filedone=1",&play_sound_types[6]},
  {BOOL_TYPE,"play_sound_privchat=1",&play_sound_types[7]},
#endif

  {NULL,NULL,NULL}
};

static void init_colors(){
  int i;

  if(initialized_colors==FALSE){
    initialized_colors=TRUE;
  } else {
    for(i=0;i<4;i++)
      gdk_colormap_free_colors(main_colormap,&userlist_colors[i],1);
  }
  for(i=0;i<4;i++)
    gdk_colormap_alloc_color(main_colormap,&userlist_colors[i],FALSE,TRUE);
}

gboolean load_prefs(){
  ConfigElement *ce;
  gnome_config_push_prefix("/" PACKAGE "/Main/Prefs/");
  for(ce=&confitems[0];ce->path!=NULL;ce++)
    ce->type->read(ce);
  gnome_config_pop_prefix();
  init_colors();
  return FALSE;
}

static void close_box(){
  property_window=NULL;
  gtk_idle_add( (gboolean (*)(gpointer) )load_prefs,NULL);
}

static void save_prefs(){
  ConfigElement *ce;
  gnome_config_push_prefix("/" PACKAGE "/Main/Prefs/");
  for(ce=&confitems[0];ce->path!=NULL;ce++)
    ce->type->write(ce);
  gnome_config_sync();
  gnome_config_pop_prefix();
}

static void get_prefs(){
  GList *r;
  if(nickname!=NULL)
    free(nickname);
  nickname=gtk_editable_get_chars(GTK_EDITABLE(nickname_entry_widget),0,-1);
  if(default_local_dir!=NULL)
    free(default_local_dir);
  default_local_dir=gtk_editable_get_chars(GTK_EDITABLE(default_local_dir_widget),0,-1);
  while(hotline_data_files!=NULL){
    free(hotline_data_files->data);
    hotline_data_files=g_list_remove(hotline_data_files,hotline_data_files->data);
  }
  for(r=GTK_CLIST(icon_files_widget)->row_list;r!=NULL;r=r->next){
    hotline_data_files=g_list_append(hotline_data_files,
				     strdup(GTK_CLIST_ROW(r)->cell[0].u.text));
  }
}
			       
static void apply_prefs(GnomePropertyBox *property_box, gint page_num){
  if(page_num!=-1) return;
  get_prefs();
  save_prefs();
}

static void boolean_state_changed(GtkWidget *w,gpointer data){
  if(property_window==NULL) return;
  *(gboolean *)data=
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
  gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}
static void integer_adjustment_changed(GtkAdjustment *w,gpointer data){
  *(gint *)data=(gint)(w->value);
  gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}
static void editable_changed(GtkEditable *w,gpointer data){
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}

static void widget_set_font(GtkWidget *w,PrefsFont *pf){
  GtkStyle *style;

  if(pf->font==NULL) return;
  style=gtk_style_copy(gtk_widget_get_style(main_app));
  gdk_font_unref(style->font);
  style->font=pf->font;
  gdk_font_ref(style->font);
  gtk_widget_set_style(w,style);
}

static void font_changed(GnomeFontPicker *fontpicker,
			      gpointer arg1,
			      gpointer user_data){
  PrefsFont *pf=(PrefsFont *)user_data;
  if(pf->font_name!=NULL)
    free(pf->font_name);
  pf->font_name=strdup(gnome_font_picker_get_font_name(fontpicker));
  if(pf->font!=NULL)
    gdk_font_unref(pf->font);
  pf->font=gnome_font_picker_get_font(fontpicker);
  gdk_font_ref(pf->font);
  gtk_label_set_text(GTK_LABEL(pf->label),pf->font_name);
  widget_set_font(pf->label,pf);
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}

static GtkWidget *create_boolean_checkbox(char *name,gboolean *value){
  GtkWidget *w=gtk_check_button_new_with_label(name);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),*value);
  gtk_signal_connect(GTK_OBJECT(w),"toggled",
		     GTK_SIGNAL_FUNC(boolean_state_changed),
		     value);
  return w;
}

static void boolean_state_changed_activate(GtkWidget *w,gpointer data){
  gtk_widget_set_sensitive(GTK_WIDGET(data),gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
}

static void boolean_checkbox_activates(GtkToggleButton *t,GtkWidget *w){
  gtk_signal_connect(GTK_OBJECT(t),"toggled",
		     GTK_SIGNAL_FUNC(boolean_state_changed_activate),
		     w);
  boolean_state_changed_activate(GTK_WIDGET(t),w);
}

static void color_picker_changed(GnomeColorPicker *colorpicker,
				 guint arg1,
				 guint arg2,
				 guint arg3,
				 guint arg4,
				 gpointer user_data){
  GdkColor *c=(GdkColor *)user_data;
  gushort *alpha,aaa;
  alpha=(guint16 *)gtk_object_get_data(GTK_OBJECT(colorpicker),"alpha_ptr");
  if(alpha==NULL)
    alpha=&aaa;
  gnome_color_picker_get_i16(colorpicker,&c->red,&c->green,&c->blue,alpha);
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}

static GtkWidget *create_color_box(char *title,GdkColor *color,gushort *alpha){
  GtkWidget *c=gnome_color_picker_new();
  GtkWidget *f;

  gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(c),alpha!=NULL?TRUE:FALSE);
  gnome_color_picker_set_dither(GNOME_COLOR_PICKER(c),TRUE);
  gnome_color_picker_set_i16(GNOME_COLOR_PICKER(c),
			     color->red,color->green,color->blue,(alpha==NULL)?0:*alpha);
  gtk_object_set_data(GTK_OBJECT(c),"alpha_ptr",alpha);
  gtk_signal_connect(GTK_OBJECT(c),"color-set",
		     GTK_SIGNAL_FUNC(color_picker_changed),
		     color);
  f=gtk_hbox_new(FALSE,0);
  gtk_box_pack_end(GTK_BOX(f),c,FALSE,FALSE,0);
  gtk_box_pack_end(GTK_BOX(f),gtk_label_new(title),FALSE,FALSE,0);

  return f;
}

static GtkWidget *create_string_entry(char *label,char **text,
				      GtkWidget **entry){
  GtkWidget *hbox=gtk_hbox_new(FALSE,0);

  *entry=gtk_entry_new();
  if(*text!=NULL)
    gtk_entry_set_text(GTK_ENTRY(*entry),*text);
  gtk_box_pack_start(GTK_BOX(hbox),gtk_label_new(label),FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(hbox),*entry,TRUE,TRUE,0);
  gtk_signal_connect(GTK_OBJECT(*entry),"changed",
		     GTK_SIGNAL_FUNC(editable_changed),
		     (gpointer)text);
  return hbox;
}

static GtkWidget *create_font_picker(PrefsFont *pf,char *title,char *use){
  GtkWidget *t=gnome_font_picker_new(),*hbox=gtk_hbox_new(FALSE,1);
  GtkWidget *frame=gtk_frame_new(title);

  pf->picker=t;
  gtk_container_add(GTK_CONTAINER(frame),hbox);
  gtk_signal_connect(GTK_OBJECT(pf->picker),"font-set",
		     GTK_SIGNAL_FUNC(font_changed),
		     (gpointer)pf);
  pf->label=gtk_label_new("aa");
  gnome_font_picker_set_title(GNOME_FONT_PICKER(pf->picker),title); 
  if(pf->font_name!=NULL)
    gnome_font_picker_set_font_name(GNOME_FONT_PICKER(pf->picker),
				    pf->font_name);
  gtk_label_set_text(GTK_LABEL(pf->label),
		     gnome_font_picker_get_font_name(GNOME_FONT_PICKER(t))); 
  widget_set_font(pf->label,pf);
  gnome_font_picker_set_mode(GNOME_FONT_PICKER(pf->picker),
			     GNOME_FONT_PICKER_MODE_USER_WIDGET);
  gnome_font_picker_uw_set_widget(GNOME_FONT_PICKER(pf->picker),pf->label);
  t=create_boolean_checkbox(use,&pf->use_custom_font);
  gtk_box_pack_start(GTK_BOX(hbox),t,TRUE,TRUE,1);
  gtk_box_pack_start(GTK_BOX(hbox),pf->picker,TRUE,TRUE,1);
  boolean_checkbox_activates(GTK_TOGGLE_BUTTON(t),pf->picker);
  gtk_widget_show(pf->picker);
  return frame;
}

void guiprefs_widget_set_font(GtkWidget *w,PrefsFont *pf){
  if(pf->use_custom_font==FALSE){
    gtk_widget_set_style(w,gtk_widget_get_style(main_app));
    return;
  }
  widget_set_font(w,pf);
}

static GtkWidget *create_adjustment(GtkAdjustment *a,int *val){
  GtkWidget *s;
  gtk_signal_connect(GTK_OBJECT(a),"value-changed",
		     GTK_SIGNAL_FUNC(integer_adjustment_changed),
		     val);
  s=gtk_hscale_new(a);
  gtk_scale_set_value_pos(GTK_SCALE(s),GTK_POS_LEFT);
  return s;
}

typedef struct {
  char *name;
  int val;
} OMSettings;

static void optionmenu_select(GtkMenuItem *m,gpointer data){
  int *var=gtk_object_get_data(GTK_OBJECT(m),"VarPtr");
  OMSettings *s=(OMSettings *)data;
  
  if(*var==s->val)
    return;
  *var=s->val;
  printf("%d %d %s\n",*var,s->val,s->name);
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}

static GtkWidget *create_optionmenu(char *name,int *var,OMSettings *s){
  GtkWidget *menu,*w,*mi;
  int i,sel=0;
  
  w=gtk_option_menu_new();
  menu=gtk_menu_new();
  for(i=0;s->name!=NULL;i++,s++){
    if(s->val==*var)
      sel=i;
    mi=gtk_menu_item_new_with_label(s->name);
    gtk_object_set_data(GTK_OBJECT(mi),"VarPtr",var);
    gtk_signal_connect(GTK_OBJECT(mi),"activate",
		       GTK_SIGNAL_FUNC(optionmenu_select),
		       (gpointer)s);
    gtk_menu_append(GTK_MENU(menu),mi);
  }
  gtk_option_menu_set_menu(GTK_OPTION_MENU(w),menu);
  gtk_option_menu_set_history(GTK_OPTION_MENU(w),sel);
  mi=gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(mi),gtk_label_new(name),FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(mi),w,TRUE,TRUE,0);
  return mi;
}

/* =========================== */

/* =========================== */

static GtkWidget *createUserPage(){
  GtkWidget *frame,*t,*t1,*table;
  GtkAdjustment *a;

  frame=gtk_vbox_new(FALSE,0);
  table=gtk_table_new(2,2,TRUE);
  gtk_box_pack_start(GTK_BOX(frame),table,FALSE,FALSE,0);
  
  t=create_string_entry(_("Nickname: "),&nickname,&nickname_entry_widget);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,2,0,1);
  t=create_boolean_checkbox(_("Append maximum speed to nickname"),&nickname_append_max_speed);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,1,2);
  a=GTK_ADJUSTMENT(gtk_adjustment_new((float)nickname_append_speed_seconds,
				      10.0,1000.0,10.0,1.0,60.0));
  t1=create_adjustment(a,&nickname_append_speed_seconds);
  gtk_table_attach_defaults(GTK_TABLE(table),t1,1,2,1,2);
  boolean_checkbox_activates(GTK_TOGGLE_BUTTON(t),t1);
  return frame;
}

static GtkWidget *createMiscPage(){
  GtkWidget *frame,*t,*hbox,*vbox;
  GtkWidget *table;
  GtkAdjustment *a;
  static OMSettings NBTS[]={
    {N_("Top"),GTK_POS_TOP},{N_("Left"),GTK_POS_LEFT},
    {N_("Bottom"),GTK_POS_BOTTOM},{N_("Right"),GTK_POS_RIGHT},
    {NULL,0}};

  frame=gtk_vbox_new(FALSE,0);
  table=gtk_table_new(6,2,TRUE);
  gtk_box_pack_start(GTK_BOX(frame),table,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Interactive connections in Bookmarks"),
			    &prefs_dblclick_connect_query_bookmarks);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,0,1);
  t=create_boolean_checkbox(_("Interactive connections in Trackers"),
			    &prefs_dblclick_connect_query_trackers);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,1,2);
  t=create_boolean_checkbox(_("Hide Passwords"),&passwords_are_hidden);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,2,3);
  t=create_boolean_checkbox(_("Ignore port number for duplicate servers in trackers"),&tracker_sort_only_by_ip);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,3,4);

  {
    GtkWidget *f1=gtk_frame_new(_("Notebook"));
    GtkWidget *tbl=gtk_table_new(2,2,TRUE);
    gtk_table_attach_defaults(GTK_TABLE(table),f1,1,2,0,3);
    gtk_container_add(GTK_CONTAINER(f1),tbl);
    t=create_optionmenu(_("Notebook tab side: "),&notebook_tab_side,NBTS);
    gtk_table_attach_defaults(GTK_TABLE(tbl),t,0,2,0,1);
    t=create_boolean_checkbox(_("Notebook tabs have icons"),&notebook_has_icons);
    gtk_table_attach_defaults(GTK_TABLE(tbl),t,0,1,1,2);
    t=create_color_box(_("Tab activity color"),
		       &notebook_tab_highlight_color,NULL);
    gtk_table_attach_defaults(GTK_TABLE(tbl),t,1,2,1,2);
  }
  {
    GtkWidget *f1=gtk_frame_new(_("Speeds"));
    hbox=gtk_hbox_new(FALSE,0);
    //  gtk_box_pack_start(GTK_BOX(frame),hbox,FALSE,FALSE,0);
    gtk_table_attach_defaults(GTK_TABLE(table),f1,1,2,3,5);
    gtk_container_add(GTK_CONTAINER(f1),hbox);
    vbox=gtk_vbox_new(TRUE,0);
    gtk_box_pack_start(GTK_BOX(hbox),vbox,FALSE,FALSE,0);
    gtk_box_pack_start(GTK_BOX(vbox),gtk_label_new(_("Task refresh speed:")),
		       TRUE,TRUE,0);
  }
  vbox=gtk_vbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(hbox),vbox,TRUE,TRUE,0);

  a=GTK_ADJUSTMENT(gtk_adjustment_new((float)prefs_task_refresh_msec,
				      10.0,1500.0,10.0,100.0,100.0));
  t=create_adjustment(a,&prefs_task_refresh_msec);
  gtk_box_pack_start(GTK_BOX(vbox),t,TRUE,TRUE,0);
  return frame;
}

static GtkWidget *createAgreementPage(){
  GtkWidget *frame,*t,*table;

  frame=gtk_vbox_new(FALSE,0);
  table=gtk_table_new(3,2,TRUE);
  gtk_box_pack_start(GTK_BOX(frame),table,FALSE,FALSE,1);
  t=create_boolean_checkbox(_("Auto-Agree Agreements"),&dont_show_agreements);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,0,1);

  t=create_font_picker(&agreement_font,
		      _("Custom Agreement Font"),_("Use custom font for the Agreement"));
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,2,1,3);

  return frame;
}


static GtkWidget *createNewsPage(){
  GtkWidget *frame,*t,*table;

  frame=gtk_vbox_new(FALSE,0);
  table=gtk_table_new(3,2,TRUE);
  gtk_box_pack_start(GTK_BOX(frame),table,FALSE,FALSE,1);
  t=create_boolean_checkbox(_("Auto-Open News on connecting"),&auto_open_news);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,0,1);
  t=create_boolean_checkbox(_("Increase the size of the message window"),
			    &news_big_messagewindow);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,1,2);
  
  t=create_font_picker(&news_font,_("Custom News Font"),
		      _("Use custom font for News"));
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,2,2,4);

  return frame;
}

static GtkWidget *createUserlistPage(){
  GtkWidget *frame,*t,*table,*box,*vbox;

  frame=gtk_vbox_new(FALSE,0);
  table=gtk_table_new(7,2,TRUE);
  gtk_box_pack_start(GTK_BOX(frame),table,FALSE,FALSE,1);
  t=create_boolean_checkbox(_("Auto-Open userlist on connecting"),
			    &auto_open_userlist);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,0,1);
  t=create_boolean_checkbox(_("Userlist is on the right"),
			    &userlist_dock_right);
  gtk_table_attach_defaults(GTK_TABLE(table),t,1,2,0,1);
  t=create_boolean_checkbox(_("Use antialiased userlist (slower)"),
			    &userlist_aa_canvas);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,1,2);
  
  box=gtk_frame_new(_("Sorting"));
  gtk_table_attach_defaults(GTK_TABLE(table),box,0,1,2,6);
  vbox=gtk_vbox_new(TRUE,0);
  gtk_container_add(GTK_CONTAINER(box),vbox);
  t=create_boolean_checkbox(_("Sort userlist by nick"),&userlist_sort_name);
  gtk_box_pack_start(GTK_BOX(vbox),t,TRUE,TRUE,0);
  t=create_boolean_checkbox(_("Group users by status"),&userlist_group_by_status);
  gtk_box_pack_start(GTK_BOX(vbox),t,TRUE,TRUE,0);
  t=create_boolean_checkbox(_("Display status icons"),&userlist_use_status_icons);
  gtk_box_pack_start(GTK_BOX(vbox),t,TRUE,TRUE,0);

  box=gtk_frame_new(_("Colors"));
  gtk_table_attach_defaults(GTK_TABLE(table),box,1,2,2,6);
  {
    GtkWidget *table=gtk_table_new(2,2,TRUE);
    gtk_container_add(GTK_CONTAINER(box),table);
    t=create_color_box(_("Normal"),&userlist_colors[NORMAL_COLOR],NULL);
    gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,0,1);
    t=create_color_box(_("Administrator"),&userlist_colors[ADMIN_COLOR],NULL);
    gtk_table_attach_defaults(GTK_TABLE(table),t,1,2,0,1);
    t=create_color_box(_("Idle"),&userlist_colors[GHOST_COLOR],NULL);
    gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,1,2);
    t=create_color_box(_("Idle Admin"),
		       &userlist_colors[ADMIN_GHOST_COLOR],NULL);
    gtk_table_attach_defaults(GTK_TABLE(table),t,1,2,1,2);
    t=create_color_box(_("Selection"),
		       &userlist_highlight_color,&userlist_highlight_color_alpha);
    gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,2,3);
  }

  box=create_font_picker(&userlist_font,_("Custom Userlist Font"),_("Use custom font"));
  gtk_table_attach_defaults(GTK_TABLE(table),box,0,2,6,8);
  
  return frame;
}

static GtkWidget *createPubchatPage(){
  GtkWidget *box,*nbf,*t,*table,*frame;

  box=gtk_vbox_new(FALSE,0);
  nbf=gtk_hbox_new(TRUE,2);
  gtk_box_pack_start(GTK_BOX(box),nbf,FALSE,FALSE,0);
  frame=gtk_frame_new(_("Public Chat"));
  gtk_box_pack_start(GTK_BOX(nbf),frame,TRUE,TRUE,0);
  table=gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(frame),table);
  t=create_boolean_checkbox(_("Auto-Open Public chat on connecting"),
			    &auto_open_pubchat);
  gtk_box_pack_start(GTK_BOX(table),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Popup Public Chat when receiving chat"),
			    &auto_popup_pubchat);
  gtk_box_pack_start(GTK_BOX(table),t,FALSE,FALSE,0);

  frame=gtk_frame_new(_("Messages"));
  gtk_box_pack_start(GTK_BOX(nbf),frame,TRUE,TRUE,0);
  table=gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(frame),table);
  t=create_boolean_checkbox(_("Message window has reply textbox"),
			    &prefs_messages_show_with_reply);
  gtk_box_pack_start(GTK_BOX(table),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("When replying, quote message"),
			    &prefs_messages_reply_quoted);
  gtk_box_pack_start(GTK_BOX(table),t,FALSE,FALSE,0);
  
  t=create_font_picker(&chat_font,_("Custom Chat Font"),_("Use custom font in Chat"));
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  
  return box;
}

static GtkWidget *createFilesPage(){
  GtkWidget *t,*table,*frame,*vbox,*box;
  GtkAdjustment *a;

  vbox=gtk_vbox_new(FALSE,0);
  table=gtk_table_new(3,2,TRUE);
  gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Auto-open Files on connecting"),
			    &auto_open_files);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,1,0,1);
  t=create_boolean_checkbox(_("Mix files and folders"),
			    &mix_files_and_folders);
  gtk_table_attach_defaults(GTK_TABLE(table),t,1,2,0,1);
  t=create_boolean_checkbox(_("Show small sizes"),
			    &files_small_sizes);
  gtk_table_attach_defaults(GTK_TABLE(table),t,1,2,1,2);
  t=create_string_entry(_("Default directory:"),&default_local_dir,
			&default_local_dir_widget);
  gtk_table_attach_defaults(GTK_TABLE(table),t,0,2,2,3);

  table=gtk_hbox_new(TRUE,1);
  gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,1);
  frame=gtk_frame_new(_("Local files"));
  gtk_box_pack_start(GTK_BOX(table),frame,TRUE,TRUE,1);
  box=gtk_vbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(frame),box);
  t=create_boolean_checkbox(_("Convert Mac TEXT -> Unix text files"),
			    &download_convert_text);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Queue Downloads"),
			    &download_queue);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Hide local dotfiles"),
			    &hide_local_dotfiles);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Strip leading/trailing spaces of filenames"),
			&strip_spaces_on_local_files);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("Sanitize local filenames"),
			&sanitize_local_filenames);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  t=create_boolean_checkbox(_("After aborting, remove zero-length files"),
			&download_remove_null_files);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  {
    GtkWidget *abox=gtk_vbox_new(FALSE,1);
    GtkWidget *tbl;
    gtk_box_pack_start(GTK_BOX(table),abox,TRUE,TRUE,1);

    frame=gtk_frame_new(_("Retry options"));
    gtk_box_pack_start(GTK_BOX(abox),frame,FALSE,FALSE,0);
    tbl=gtk_table_new(2,2,FALSE);
    gtk_container_add(GTK_CONTAINER(frame),tbl);
    gtk_table_attach(GTK_TABLE(tbl),
		     t=gtk_label_new(_("Retry count:")),
		     0,1,0,1,0,0,0,0);
    gtk_misc_set_alignment(GTK_MISC(t),1.0,0.5);
    a=GTK_ADJUSTMENT(gtk_adjustment_new((float)retry_count,
					0.0,100.0,1.0,10.0,10.0));
    t=create_adjustment(a,&retry_count);
    gtk_table_attach_defaults(GTK_TABLE(tbl),t,
			      1,2,0,1);
    gtk_table_attach(GTK_TABLE(tbl),
		     t=gtk_label_new(_("Retry delay:")),
		     0,1,1,2,0,0,0,0);
    gtk_misc_set_alignment(GTK_MISC(t),1.0,0.5);
    a=GTK_ADJUSTMENT(gtk_adjustment_new((float)retry_delay,
					0.0,100.0,1.0,10.0,10.0));
    t=create_adjustment(a,&retry_delay);
    gtk_table_attach_defaults(GTK_TABLE(tbl),t,
			      1,2,1,2);
  
    frame=gtk_frame_new(_("Remote files"));
    gtk_box_pack_start(GTK_BOX(abox),frame,TRUE,TRUE,1);
    box=gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(frame),box);
    t=create_boolean_checkbox(_("Convert Unix text -> Mac TEXT files"),
			      &upload_convert_text);
    gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
    t=create_boolean_checkbox(_("Queue Uploads"),
			      &upload_queue);
    gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
    t=create_boolean_checkbox(_("Cache remote listings"),
			      &cache_remote_directories);
    gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,0);
  }
  return vbox;
}

static GtkWidget *progbar=NULL,*icl=NULL,*licl;

static void update_icon(gpointer data){
  int icon=GPOINTER_TO_INT(data);
  Icon *ic;
  if(icon==user_icon_number) return;
  ic=get_icon(icon);
  user_icon_number=icon;
  gtk_pixmap_set(GTK_PIXMAP(icon_pixmap_widget),ic->pixmap,ic->mask);
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}
static void set_small_icon(GnomeIconList *iconlist,
			   gint arg1,
			   GdkEvent *event,
			   gpointer user_data){
  if(event==NULL) return;
  gtk_clist_unselect_all(GTK_CLIST(licl));
  update_icon(gnome_icon_list_get_icon_data(GNOME_ICON_LIST(icl),arg1));
}

static void set_big_icon(GtkCList *clist,
			 gint row,
			 gint column,
			 GdkEventButton *event,
			 gpointer user_data){
  if(event==NULL) return;
  gnome_icon_list_unselect_all(GNOME_ICON_LIST(icl),NULL,NULL);
  update_icon(gtk_clist_get_row_data(GTK_CLIST(licl),row));
}

static void add_icon(Icon *icon,gpointer data){
  int i;
  icon=get_icon(icon->number);
  if(icon==NULL) return;
  i=gnome_icon_list_append_imlib(GNOME_ICON_LIST(data),icon->image,
				 icon->name);
  gnome_icon_list_set_icon_data(GNOME_ICON_LIST(data),i,
				GINT_TO_POINTER(icon->number));
}

static void add_big_icon(Icon *icon,gpointer data){
  int i;
  char *ptr[5];
  char buf[20];
  icon=get_icon(icon->number);
  if(icon==NULL) return;
  ptr[2]="";ptr[1]=icon->name;ptr[0]=buf;
  sprintf(buf,"%5d",icon->number);
  i=gtk_clist_append(GTK_CLIST(licl),ptr);
  gtk_clist_set_row_data(GTK_CLIST(licl),i,GINT_TO_POINTER(icon->number));
  gtk_clist_set_pixmap(GTK_CLIST(licl),i,2,icon->pixmap,icon->mask);
}

static void update_icons(GtkWidget *but,gpointer data){
  int i;
  load_all_icons(progbar);
  gnome_icon_list_freeze(GNOME_ICON_LIST(icl));
  gnome_icon_list_clear(GNOME_ICON_LIST(icl));
  for_all_icons(TRUE,add_icon,icl);
  gnome_icon_list_thaw(GNOME_ICON_LIST(icl));
  gtk_clist_freeze(GTK_CLIST(licl));
  for_all_icons(FALSE,add_big_icon,licl);
  gtk_clist_thaw(GTK_CLIST(licl));
  i=gnome_icon_list_find_icon_from_data(GNOME_ICON_LIST(icl),
					GINT_TO_POINTER(user_icon_number));
  if(i!=-1){
    gnome_icon_list_select_icon(GNOME_ICON_LIST(icl),i);
    gnome_icon_list_moveto(GNOME_ICON_LIST(icl),i,0.5);
  }
  i=gtk_clist_find_row_from_data(GTK_CLIST(licl),
				 GINT_TO_POINTER(user_icon_number));
  if(i!=-1){
    gtk_clist_select_row(GTK_CLIST(licl),i,0);
    gtk_clist_moveto(GTK_CLIST(licl),i,0,0.5,0.0);
  }
}

static void add_data_file(GtkWidget *button,GnomeFileEntry *f){
  GtkWidget *e=gnome_file_entry_gtk_entry(f);
  char *a=gtk_editable_get_chars(GTK_EDITABLE(e),0,-1);
  if(a==NULL || strlen(a)==0) return;
  gtk_clist_append(GTK_CLIST(icon_files_widget),&a);
  g_free(a);
  gtk_entry_set_text(GTK_ENTRY(e),"");
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}
static void remove_data_file(GtkWidget *button,GnomeFileEntry *f){
  GList *l;
  char *m;
  GtkWidget *e=gnome_file_entry_gtk_entry(f);
  int r;

  if((l=GTK_CLIST(icon_files_widget)->selection)==NULL) return;
  r=GPOINTER_TO_INT(l->data);
  m=gtk_clist_get_row_data(GTK_CLIST(icon_files_widget),r);
  if(m!=NULL)
    gtk_entry_set_text(GTK_ENTRY(e),m);
  else {
    GtkCListRow *row=g_list_nth(GTK_CLIST(icon_files_widget)->row_list,r)->data;
    gtk_entry_set_text(GTK_ENTRY(e),row->cell[0].u.text);
  }
  gtk_clist_remove(GTK_CLIST(icon_files_widget),r);
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}
static void move_data_file(GtkCList *clist,
			   gint arg1,
			   gint arg2,
			   gpointer user_data){
  if(property_window!=NULL)
    gnome_property_box_changed(GNOME_PROPERTY_BOX(property_window));
}

static GtkWidget *createIconPage(){
  GtkWidget *vbox,*box,*t,*nt,*o,*use_icons;
  Icon *icon;
  char *titles[]={_("Icon Number"),_("Icon Name"),_("Icon"),""};

  vbox=gtk_vbox_new(FALSE,0);
  box=gtk_hbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(vbox),box,FALSE,FALSE,0);
  use_icons=t=create_boolean_checkbox(_("Use Icons"),&userlist_use_icons);
  gtk_box_pack_start(GTK_BOX(box),t,TRUE,TRUE,0);
  t=create_boolean_checkbox(_("Use Hotline icons in buttons and menus"),&use_hotline_icons);
  gtk_box_pack_start(GTK_BOX(box),t,TRUE,TRUE,0);
  boolean_checkbox_activates(GTK_TOGGLE_BUTTON(use_icons),t);

  box=gtk_hbox_new(FALSE,0);
  boolean_checkbox_activates(GTK_TOGGLE_BUTTON(use_icons),box);
  gtk_box_pack_start(GTK_BOX(vbox),box,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(box),gtk_label_new(_("Icon:")),FALSE,FALSE,2);
  icon=get_icon(user_icon_number);
  if(icon!=NULL){
    icon_pixmap_widget=gtk_pixmap_new(icon->pixmap,icon->mask);
  } else {
    icon_pixmap_widget=gtk_pixmap_new(NULL,NULL);
  }
  gtk_box_pack_start(GTK_BOX(box),icon_pixmap_widget,FALSE,FALSE,2);
  t=gtk_button_new_with_label(_("Load Icons"));
  gtk_signal_connect(GTK_OBJECT(t),"clicked",
		     GTK_SIGNAL_FUNC(update_icons),NULL);
  gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,2);
  progbar=gtk_progress_bar_new();
  gtk_box_pack_start(GTK_BOX(box),progbar,TRUE,TRUE,2);

  nt=gtk_notebook_new();
  boolean_checkbox_activates(GTK_TOGGLE_BUTTON(use_icons),nt);
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(nt),GTK_POS_LEFT);
  gtk_box_pack_start(GTK_BOX(vbox),nt,TRUE,TRUE,0);
  box=gtk_scrolled_window_new(NULL,NULL);
  icl=gnome_icon_list_new_flags(100,NULL,GNOME_ICON_LIST_STATIC_TEXT);
  gtk_signal_connect(GTK_OBJECT(icl),"select-icon",GTK_SIGNAL_FUNC(set_small_icon),NULL);
  gtk_container_add(GTK_CONTAINER(box),icl);
  gtk_notebook_append_page(GTK_NOTEBOOK(nt),box,gtk_label_new(_("Small Icons")));
  box=gtk_scrolled_window_new(NULL,NULL);
  licl=gtk_clist_new_with_titles(3,titles);
  gtk_container_add(GTK_CONTAINER(box),licl);
  gtk_signal_connect(GTK_OBJECT(licl),"select-row",GTK_SIGNAL_FUNC(set_big_icon),NULL);
  gtk_clist_set_row_height(GTK_CLIST(licl),18);
  gtk_notebook_append_page(GTK_NOTEBOOK(nt),box,gtk_label_new(_("Large Icons")));

  o=gtk_vbox_new(FALSE,0);
  {
    GtkWidget *hbox,*vbox,*entry;
    GList *l;
    int row;

    hbox=gtk_hbox_new(0,FALSE);
    gtk_box_pack_start(GTK_BOX(o),hbox,TRUE,TRUE,0);
    vbox=gtk_vbox_new(0,FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),vbox,TRUE,TRUE,0);
    entry=gnome_file_entry_new(NULL,_("Datafile"));
    gtk_box_pack_end(GTK_BOX(vbox),entry,FALSE,FALSE,0);
    icon_files_widget=gtk_clist_new(1);
    gtk_signal_connect(GTK_OBJECT(icon_files_widget),"row-move",
		       GTK_SIGNAL_FUNC(move_data_file),entry);
    gtk_clist_set_reorderable(GTK_CLIST(icon_files_widget),TRUE);
    t=gtk_scrolled_window_new(NULL,NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(t),
				   GTK_POLICY_NEVER,
				   GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(t),icon_files_widget);
    gtk_box_pack_start(GTK_BOX(vbox),t,TRUE,TRUE,0);
    vbox=gtk_vbox_new(0,FALSE);
    gtk_box_pack_start(GTK_BOX(hbox),vbox,FALSE,FALSE,0);
    gtk_box_pack_end(GTK_BOX(vbox),t=gtk_button_new_with_label(_("Add")),FALSE,FALSE,0);
    gtk_signal_connect(GTK_OBJECT(t),"clicked",GTK_SIGNAL_FUNC(add_data_file),entry);
    gtk_box_pack_end(GTK_BOX(vbox),t=gtk_button_new_with_label(_("Remove")),FALSE,FALSE,0);
    gtk_signal_connect(GTK_OBJECT(t),"clicked",GTK_SIGNAL_FUNC(remove_data_file),entry);
    for(l=hotline_data_files;l!=NULL;l=l->next){
      row=gtk_clist_append(GTK_CLIST(icon_files_widget),(char **)&l->data);
      gtk_clist_set_row_data(GTK_CLIST(icon_files_widget),row,l->data);
    }
  }
  gtk_notebook_prepend_page(GTK_NOTEBOOK(nt),o,gtk_label_new(_("Icon Datafiles")));
  return vbox;
}

#ifdef HAVE_ESD
gboolean guiprefs_check_play_sound(int id){
  return (play_sounds && play_sound_types[id-HL_SOUND_CHAT])?TRUE:FALSE;
}

static GtkWidget *createSoundPage(){
  GtkWidget *frame,*t,*vbox,*box;
  int i;
  static struct {int id;char *name;} sound_names[]={
    {HL_SOUND_LOGGED_IN,N_("Successful login")},
    {HL_SOUND_LOGIN,    N_("User login")},
    {HL_SOUND_LOGOUT,   N_("User logout")},
    {HL_SOUND_FILEDONE, N_("File transfer complete")},
    {HL_SOUND_CHAT,     N_("Public chat message")},
    {HL_SOUND_SERVMSG,  N_("Broadcast server message")},
    {HL_SOUND_DOORBELL, N_("Private chat requested")},
    {HL_SOUND_ERROR,    N_("Server error")},
    {0,NULL}
  };

  vbox=gtk_vbox_new(FALSE,0);
  t=create_boolean_checkbox(_("Play Sounds"),&play_sounds);
  gtk_box_pack_start(GTK_BOX(vbox),t,FALSE,FALSE,1);
  frame=gtk_frame_new(_("Play sounds for these events:"));
  boolean_checkbox_activates(GTK_TOGGLE_BUTTON(t),frame);
  gtk_box_pack_start(GTK_BOX(vbox),frame,TRUE,TRUE,1);
  t=gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(t),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  box=gtk_vbox_new(FALSE,0);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(t),box);
  gtk_container_add(GTK_CONTAINER(frame),t);  
  for(i=0;sound_names[i].name!=NULL;i++){
    t=create_boolean_checkbox(sound_names[i].name,&play_sound_types[sound_names[i].id-HL_SOUND_CHAT]);
    gtk_box_pack_start(GTK_BOX(box),t,FALSE,FALSE,1);
  }
  return vbox;
}
#endif

void open_prefs(){
  static PrefsPages ppages[]={
    {N_("User Info"),createUserPage},
    {N_("Icons"),createIconPage},
#ifdef HAVE_ESD
    {N_("Sound"),createSoundPage},
#endif
    {N_("Misc"),createMiscPage},
    {N_("Agreement"),createAgreementPage},
    {N_("Files"),createFilesPage},
    {N_("News"),createNewsPage},
    {N_("Userlist"),createUserlistPage},
    {N_("Chat and Messages"),createPubchatPage},
    {NULL,NULL}
  };
  int i;
  GtkWidget *w;
  GtkWidget *pw=gnome_property_box_new();
  for(i=0;ppages[i].create!=NULL;i++){
    w=ppages[i].create();
    if(w!=NULL)
      gnome_property_box_append_page(GNOME_PROPERTY_BOX(pw),w,
				     gtk_label_new(ppages[i].name));
  }
  gtk_signal_connect(GTK_OBJECT(pw),"apply",
		     GTK_SIGNAL_FUNC(apply_prefs),NULL);
  gtk_signal_connect(GTK_OBJECT(pw),"destroy",
		     GTK_SIGNAL_FUNC(close_box),NULL); 
  gtk_widget_show_all(pw);
  property_window=pw;
  return;
}

