/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnome.h>

#include "tasks.h"
#include "server.h"
#include "protocol.h"
#include "guiprefs.h"
#include "pubchat.h"
#include "guiutils.h"
#include "userlist.h"
#include "userwidget.h"
#include "hldat.h"

#define PUBCHAT "Public Chat"

typedef struct {
  Connection *c;
  NotebookPage *notebook_page;
  UserWidget *user_list;
  GtkWidget *entry_widget;
  GtkWidget *text_widget;
  char *send_message;
  gboolean as_emote;
} PubchatData;


static void pubchat_destroy(Connection *c,gpointer data){
  PubchatData *p=(PubchatData *)data;
  hooks_set_data(p->c,PUBCHAT,NULL);
  if(p->user_list!=NULL)
    userwidget_destroy(p->user_list);
  if(p->notebook_page!=NULL)
    gutils_nbpage_destroy(p->notebook_page);
  p->notebook_page=NULL;
  p->entry_widget=NULL;
  p->text_widget=NULL;
  if(p->send_message!=NULL)
    free(p->send_message);
  free(p);
}

static void pubchat_clear_text(GtkButton *b,gpointer data){
  PubchatData *p=(PubchatData *)data;
  if(p->text_widget!=NULL)
    gtk_editable_delete_text(GTK_EDITABLE(p->text_widget),0,-1);
}

static void pubchat_send_message(Connection *c,gpointer data){
  PubchatData *p=(PubchatData *)data;
  HLTransaction *t;
  char *m;

  if(p==NULL || p->send_message==NULL)
    return;
  t=transaction_new(HLCI_SENDCHAT,NULL,FALSE);
  transaction_add_object(t,create_string(HLO_MESSAGE,p->send_message));
  if(p->as_emote)
    transaction_add_object(t,create_number(HLO_PARAMETER,1));
  transaction_send(t,p->c);
  transaction_destroy(t);
  m=p->send_message;
  p->send_message=NULL;
  free(m);
}

static void pubchat_send_text(GtkEditable *editable,gpointer data){
  PubchatData *p=(PubchatData *)data;
  if(p->entry_widget==NULL || p->send_message!=NULL)
    return;
  p->send_message=gtk_editable_get_chars(GTK_EDITABLE(p->entry_widget),0,-1);
  p->as_emote=FALSE;
  server_transaction_start(p->c,pubchat_send_message,p);
  gtk_editable_delete_text(GTK_EDITABLE(p->entry_widget),0,-1);
}

static void pubchat_send_emote(GtkButton *editable,gpointer data){
  PubchatData *p=(PubchatData *)data;
  if(p->entry_widget==NULL || p->send_message!=NULL)
    return;
  p->send_message=gtk_editable_get_chars(GTK_EDITABLE(p->entry_widget),0,-1);
  p->as_emote=TRUE;
  server_transaction_start(p->c,pubchat_send_message,p);
  gtk_editable_delete_text(GTK_EDITABLE(p->entry_widget),0,-1);
}

static GnomeUIInfo pubchat_toolbar[] = {
  GNOMEUIINFO_ITEM_STOCK("Close",N_("Close public chat"),
			 pubchat_destroy,
			 GNOME_STOCK_PIXMAP_EXIT),
  GNOMEUIINFO_ITEM_STOCK("Clear",N_("Clear window"),
			 pubchat_clear_text,
			 GNOME_STOCK_PIXMAP_CLEAR),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK("Send",N_("Send this message"),
			 pubchat_send_message,
			 GNOME_STOCK_PIXMAP_MAIL_SND),
  GNOMEUIINFO_ITEM_STOCK("Emote",N_("Send this message as an emote"),
			 pubchat_send_emote,
			 GNOME_STOCK_PIXMAP_MAIL_SND),
  GNOMEUIINFO_END
};


static void pubchat_create_gui(PubchatData *p){
  GtkWidget *vbox,*text,*but,*entry,*hbox,*sl;

  if(p->notebook_page!=NULL){
    gutils_nbpage_to_front(p->notebook_page);
    return;
  }

  p->notebook_page=gutils_nbpage_new("Main",pubchat_toolbar,p->c,
				     _("Chat"),"pubchat",p);
  vbox=gtk_vbox_new(FALSE,1);

  sl=gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sl),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_ALWAYS);
  text=gtk_text_new(NULL,NULL);
  gtk_text_set_editable(GTK_TEXT(text),FALSE);
  gtk_text_set_word_wrap(GTK_TEXT(text),TRUE);
  guiprefs_widget_set_font(text,&chat_font);
  gtk_container_add(GTK_CONTAINER(sl),text);
  gtk_box_pack_start(GTK_BOX(vbox),sl,TRUE,TRUE,1);
  hbox=gtk_hbox_new(FALSE,1);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,1);
  but=gtk_button_new_with_label(_("Close"));
  gtk_box_pack_start(GTK_BOX(hbox),but,FALSE,FALSE,1);
  gtk_signal_connect(GTK_OBJECT(but),"clicked",
		     GTK_SIGNAL_FUNC(pubchat_destroy),(gpointer)p);
  but=gtk_button_new_with_label(_("Clear"));
  gtk_box_pack_start(GTK_BOX(hbox),but,FALSE,FALSE,1);
  gtk_signal_connect(GTK_OBJECT(but),"clicked",
		     GTK_SIGNAL_FUNC(pubchat_clear_text),(gpointer)p);
  entry=gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox),entry,TRUE,TRUE,1);
  gtk_signal_connect(GTK_OBJECT(entry),"activate",
		     GTK_SIGNAL_FUNC(pubchat_send_text),(gpointer)p);
  but=gtk_button_new_with_label(_("Send"));
  gtk_box_pack_start(GTK_BOX(hbox),but,FALSE,FALSE,1);
  gtk_signal_connect(GTK_OBJECT(but),"clicked",
		     GTK_SIGNAL_FUNC(pubchat_send_text),(gpointer)p);
  p->text_widget=text;
  p->entry_widget=entry;
  gutils_nbpage_set_main_with_userlist(p->notebook_page,
				       vbox,&p->user_list);
  userwidget_lists_userlist(p->user_list);
}

void check_pubchat(Connection *c){
  PubchatData *p;
  hooks_create(c,PUBCHAT,NULL,pubchat_destroy);
  p=(PubchatData *)hooks_get_data(c,PUBCHAT);
  if(p==NULL){
    p=(PubchatData *)malloc(sizeof(PubchatData));
    p->c=c;
    p->notebook_page=NULL;
    p->entry_widget=NULL;
    p->text_widget=NULL;
    p->send_message=NULL;
    p->user_list=NULL;
    hooks_set_data(c,PUBCHAT,p);
  }
  if(auto_open_pubchat==TRUE)
    pubchat_create_gui(p);
}

void want_pubchat(Connection *c){
  PubchatData *p=(PubchatData *)hooks_get_data(c,PUBCHAT);
  if(p==NULL)
    check_pubchat(c);
  p=(PubchatData *)hooks_get_data(c,PUBCHAT);
  pubchat_create_gui(p);
}

static gboolean pubchat_handle_received_mesasage(gpointer data){
  HLTransaction *t=(HLTransaction *)data;
  Connection *c=(Connection *)t->data;
  PubchatData *p=(PubchatData *)hooks_get_data(c,PUBCHAT);
  HLObject *m=transaction_find_object(t,HLO_MESSAGE);

  if((p==NULL || p->text_widget==NULL) && auto_popup_pubchat==TRUE){
    want_pubchat(c);
    p=(PubchatData *)hooks_get_data(c,PUBCHAT);
  }
  if(p!=NULL && m!=NULL && m->data.string!=NULL){
    play_sound(HL_SOUND_CHAT);
    gtk_text_insert(GTK_TEXT(p->text_widget),NULL,NULL,NULL,m->data.string,-1);
    gtk_text_insert(GTK_TEXT(p->text_widget),NULL,NULL,NULL,"\n",-1);
    guiutils_nbpage_highlight(p->notebook_page);
  }
  transaction_destroy(t);
  return FALSE;
}


void pubchat_receive_mesasage(Connection *c,HLTransaction *t,gpointer data){
  transaction_read_objects(c,t);
  t->data=c;
  gtk_idle_add(pubchat_handle_received_mesasage,t);
}
