/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <errno.h>

#include <gtk/gtk.h>
#include <gnome.h>

#include "server.h"
#include "protocol.h"
#include "guiprefs.h"
#include "smalltrans.h"
#include "userlist.h"
#include "login.h"
#include "main.h"
#include "users.h"
#include "hldat.h"

/* ================================= */
/* ================================= */
/* ================================= */

#define AGREEMENT "Agreement"

typedef struct {
  char *text;
  Connection *c;
  GtkWidget *window;
} Agreement;

static void agreement_destroy(Connection *c,gpointer p){
  Agreement *a=(Agreement *)p;
  if(a==NULL)
    return;
  free(a->text);
  if(a->window!=NULL)
    gtk_widget_destroy(a->window);
  free(a);
  hooks_set_data(c,AGREEMENT,NULL);
}

static gboolean close_agreement(GtkWidget *b,gpointer p){
  Agreement *a=(Agreement *)p;

  if(a->window!=NULL)
    gtk_widget_destroy(a->window);
  a->window=NULL;
  return FALSE;
}

static gint close_agreement_window(GtkWidget *widget,gpointer p){
  Agreement *a=(Agreement *)p;
  a->window=NULL;
  return FALSE;
}

static void show_agreement_dialog(Agreement *a,int all_buttons){
  GtkWidget *w,*text,*scrol,*parent=NULL;

  if(a->c->gui!=NULL)
    parent=a->c->gui->main_window;
  if(all_buttons==TRUE){
    w=gnome_dialog_new(_("Agreement"),_("Decline"),_("Accept"),NULL);
    gnome_dialog_button_connect(GNOME_DIALOG(w),1,
				GTK_SIGNAL_FUNC(close_agreement),
				(gpointer)a);
  } else {
    w=gnome_dialog_new(_("Agreement"),GNOME_STOCK_BUTTON_CLOSE,NULL);
  }
  a->window=w;
  gnome_dialog_button_connect(GNOME_DIALOG(w),0,
			      GTK_SIGNAL_FUNC(close_agreement),
			      (gpointer)a);
  if(parent!=NULL)
    gnome_dialog_set_parent(GNOME_DIALOG(w),GTK_WINDOW(parent));
  scrol=gtk_scrolled_window_new(NULL,NULL);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(w)->vbox),scrol,TRUE,TRUE,0);

  text=gtk_text_new(NULL,NULL);
  gtk_container_add(GTK_CONTAINER(scrol),text);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrol),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_ALWAYS);
  gtk_text_set_editable(GTK_TEXT(text),FALSE);
  gtk_text_set_word_wrap(GTK_TEXT(text),TRUE);
  gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,a->text,-1);
  guiprefs_widget_set_font(text,&agreement_font);
  
  gtk_widget_set_usize(w,350,450);
  gnome_dialog_close_hides(GNOME_DIALOG(w),FALSE);
  gtk_signal_connect(GTK_OBJECT(w),"destroy",
		     GTK_SIGNAL_FUNC(close_agreement_window),(gpointer)a);
  guiprefs_add_window(GTK_WINDOW(w),"Main/Agreement/Size");
  gtk_widget_show_all(w);  
}

void show_agreement(Connection *c){
  Agreement *a=(Agreement *)hooks_get_data(c,AGREEMENT);
  if(a==NULL || a->window!=NULL)
    return;
  show_agreement_dialog(a,FALSE);
}

static gboolean attach_agreement(gpointer data){
  Connection *c=(Connection *)data;
  Agreement *a=(Agreement *)hooks_get_data(c,AGREEMENT);

  if(a->c->gui!=NULL)
    gtk_widget_set_sensitive(a->c->gui->toolbar_info[0].widget,TRUE);
  if(dont_show_agreements==FALSE)
    show_agreement_dialog(a,TRUE);
  return FALSE;
}

void handle_server_agreement(Connection *c,HLTransaction *t,gpointer data){
  HLObject *o;
  Agreement *a=(Agreement *)malloc(sizeof(Agreement));

  a->c=c;
  a->window=NULL;
  transaction_read_objects(c,t);
  o=transaction_find_object(t,HLO_MESSAGE);
  if(o==NULL){
    free(a);
    printf(_("Whops! agreement without text!\n"));
    transaction_destroy(t);
    return;
  }
  string_net_to_unix(o);
  a->text=strdup(o->data.string);
  transaction_destroy(t);  
  hooks_create(c,AGREEMENT,NULL,agreement_destroy);
  hooks_set_data(c,AGREEMENT,(gpointer)a);
  gtk_idle_add(attach_agreement,(gpointer)c);
}

/* ================================= */
/* ================================= */
/* ================================= */

#define MESSAGES "Messages"

typedef struct {
  Connection *c;
  GtkWidget *w;
  char *message;
  gboolean is_error;
} UnkMessage;

typedef struct {
  Connection *c;
  GList *windows;
} MessageData;

typedef struct {
  MessageData *md;
  GtkWidget *toplevel;
  GtkWidget *text_widget;
  gpointer data;
  int socket;
  char *text;
} MessageWindow;

static void message_destroy_event(GtkWidget *w,gpointer data){
  MessageWindow *m=(MessageWindow *)data;
  m->md->windows=g_list_remove(m->md->windows,m);
  if(m->toplevel!=NULL && m->toplevel!=w){
    GtkWidget *d=m->toplevel;
    m->toplevel=NULL;
    gtk_signal_disconnect_by_func(GTK_OBJECT(d),GTK_SIGNAL_FUNC(message_destroy_event),m);
    gtk_widget_destroy(d);
  }
  if(m->text!=NULL)
    free(m->text);
  free(m);
}

static void message_destroy_all(Connection *c,gpointer data){
  MessageData *md=(MessageData *)hooks_get_data(c,MESSAGES);
  hooks_set_data(c,MESSAGES,NULL);
  while(md->windows!=NULL)
    message_destroy_event(NULL,md->windows->data);
}

void check_messages(Connection *c){
  MessageData *md;
  hooks_create(c,MESSAGES,NULL,message_destroy_all);
  if(hooks_get_data(c,MESSAGES)!=NULL)
    return;
  md=(MessageData *)calloc(sizeof(MessageData),1);
  md->c=c;
  md->windows=NULL;
  hooks_set_data(c,MESSAGES,md);
}

static MessageWindow *message_create_messagewindow(Connection *c,int socket){
  MessageWindow *m=(MessageWindow *)calloc(sizeof(MessageWindow),1);
  MessageData *md=(MessageData *)hooks_get_data(c,MESSAGES);

  m->md=md;
  m->toplevel=NULL;
  m->socket=socket;
  m->toplevel=NULL;
  m->text_widget=NULL;
  m->text=NULL;
  md->windows=g_list_prepend(md->windows,m);
  return m;
}

void message_display_text(Connection *c,char *string){
  MessageWindow *m=message_create_messagewindow(c,0);
  GtkWidget *label,*d;

  label=gtk_label_new(string);
  d=gnome_dialog_new(_("User Info"),GNOME_STOCK_BUTTON_CLOSE,NULL);
  if(c!=NULL && c->gui!=NULL && c->gui->main_window!=NULL)
    gnome_dialog_set_parent(GNOME_DIALOG(d),GTK_WINDOW(c->gui->main_window));
  gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),label,TRUE,TRUE,0);
  m->toplevel=d;
  gtk_signal_connect(GTK_OBJECT(d),"destroy",
		     GTK_SIGNAL_FUNC(message_destroy_event),
		     (gpointer)m);
  gnome_dialog_button_connect(GNOME_DIALOG(d),0,
			      GTK_SIGNAL_FUNC(message_destroy_event),
			      (gpointer)m);
  gtk_widget_show_all(GTK_WIDGET(d));
}

static gboolean message_transaction_send_done(gpointer data){
  message_destroy_event(NULL,data);
  return FALSE;
}

static void message_transaction_send(Connection *c,gpointer data){
  MessageWindow *m=(MessageWindow *)data;
  HLTransaction *t=transaction_new(HLCT_SENDPRIVM,NULL,FALSE);
  HLObject *o;

  transaction_add_object(t,create_number(HLO_SOCKET,m->socket));
  o=create_string(HLO_MESSAGE,m->text);
  string_unix_to_net(o);
  transaction_add_object(t,o);
  server_transaction_reply_set(m->md->c,t,server_reply_default,
			       strdup(_("Could not send Message:")));
  transaction_send(t,m->md->c);
  gtk_idle_add(message_transaction_send_done,m);
}

static void message_send(GtkButton *b,gpointer data){
  MessageWindow *m=(MessageWindow *)data;
  if(m->toplevel==NULL || m->text_widget==NULL || m->text!=NULL)
    return;
  m->text=gtk_editable_get_chars(GTK_EDITABLE(m->text_widget),0,-1);
  if(m->toplevel!=NULL)
    gtk_widget_hide(m->toplevel);
  server_transaction_start(m->md->c,message_transaction_send,m);
}

gpointer message_create_simple(Connection *c,char *message,int socket){
  MessageWindow *m=message_create_messagewindow(c,socket);
  GtkWidget *d,*text;
  UserlistEntry *ue=get_user_by_socket(c,socket);
  char buf[256];

  if(ue==NULL)
    strcpy(buf,_("Message"));
  else 
    sprintf(buf,_("Message to %s"),ue->name);

  d=gnome_dialog_new(buf,GNOME_STOCK_BUTTON_CANCEL,GNOME_STOCK_BUTTON_OK,NULL);
  gtk_signal_connect(GTK_OBJECT(d),"destroy",
		     GTK_SIGNAL_FUNC(message_destroy_event),
		     (gpointer)m);
  gnome_dialog_button_connect(GNOME_DIALOG(d),0,
			      GTK_SIGNAL_FUNC(message_destroy_event),
			      (gpointer)m);
  gnome_dialog_button_connect(GNOME_DIALOG(d),1,
			      GTK_SIGNAL_FUNC(message_send),
			      (gpointer)m);
  
  text=gtk_text_new(NULL,NULL);
  if(message!=NULL){
    gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,message,-1);
    gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,"\n\n",1);
  }
  gtk_text_set_editable(GTK_TEXT(text),TRUE);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),text,TRUE,TRUE,0);
  gtk_signal_connect(GTK_OBJECT(text),"activate",
		     GTK_SIGNAL_FUNC(message_send),
		     (gpointer)m);
  m->toplevel=d;
  m->text_widget=text;
  if(m->md->c->gui->main_window!=NULL)
    gnome_dialog_set_parent(GNOME_DIALOG(d),
			    GTK_WINDOW(m->md->c->gui->main_window));
  gtk_window_set_focus(GTK_WINDOW(d),m->text_widget);
  gtk_window_set_modal(GTK_WINDOW(d),FALSE);
  guiprefs_add_window(GTK_WINDOW(d),"Server/Messages/ReplyDialogSize");
  gtk_widget_show_all(d);
  return m;
}

static void message_clear_window(GtkButton *b,gpointer data){
  MessageWindow *m=(MessageWindow *)data;
  if(m->text_widget!=NULL)
    gtk_editable_delete_text(GTK_EDITABLE(m->text_widget),0,-1);
}

static void message_open_reply(GtkButton *b,gpointer data){
  MessageWindow *m=(MessageWindow *)data;
  message_create_simple(m->md->c,NULL,m->socket);
  message_destroy_event(NULL,m);
}

static void message_quote_and_fill(char *msg, GtkWidget *text){
  char **v=g_strsplit(msg,"\n",0);
  int i;

  gtk_text_freeze(GTK_TEXT(text));
  if(v!=NULL)
    for(i=0;v[i]!=NULL;i++){
      gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,"> ",2);
      gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,v[i],-1);
      gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,"\n",1);
    }
  gtk_text_insert(GTK_TEXT(text),NULL,NULL,NULL,"\n",1);
  g_strfreev(v);
  gtk_text_thaw(GTK_TEXT(text));
}

static void message_open_reply_quoted(GtkButton *t,gpointer data){
  MessageWindow *m=(MessageWindow *)data,*b;
  char *p;
  b=(MessageWindow *)message_create_simple(m->md->c,NULL,m->socket);
  p=gtk_editable_get_chars(GTK_EDITABLE(m->text_widget),0,-1);
  message_quote_and_fill(p,b->text_widget);
  free(p);
  message_destroy_event(NULL,m);
}

static MessageWindow *message_handle_privmsg(Connection *c,HLTransaction *t,
					     int socket,char *nick,
					     char *origmsg){
  MessageWindow *m;
  char buf[256];
  GtkWidget *vbox,*sl;
  GtkWidget *text_message,*paned=NULL;
  int i=0;
  UserlistEntry *ue;

  if(users_ignored(c,socket)==TRUE) return NULL;
  m=message_create_messagewindow(c,socket);
  ue=get_user_by_socket(c,socket);
  if((ue!=NULL && ue->name!=NULL) || nick!=NULL)
    sprintf(buf,_("Message from %s"),(ue!=NULL && ue->name!=NULL)?get_user_by_socket(c,socket)->name:nick);
  else
    sprintf(buf,_("Message from socket %d ???"),socket);
  if(prefs_messages_show_with_reply==TRUE){
    GtkWidget *text_reply;
    i=1;
    m->toplevel=gnome_dialog_new(buf,_("Clear"),_("Reply"),
				 GNOME_STOCK_BUTTON_CLOSE,NULL);
    vbox=GNOME_DIALOG(m->toplevel)->vbox;
    paned=gtk_vpaned_new();
    gtk_box_pack_end(GTK_BOX(vbox),paned,TRUE,TRUE,2);
    text_reply=gtk_text_new(NULL,NULL);
    gtk_text_set_editable(GTK_TEXT(text_reply),TRUE);
    sl=gtk_scrolled_window_new(NULL,NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sl),
				   GTK_POLICY_NEVER,
				   GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(sl),text_reply);
    gtk_paned_pack2(GTK_PANED(paned),sl,TRUE,FALSE);
    if(prefs_messages_reply_quoted==TRUE)
      message_quote_and_fill(origmsg,text_reply);
    m->text_widget=text_reply;
    gnome_dialog_button_connect(GNOME_DIALOG(m->toplevel),0,
				GTK_SIGNAL_FUNC(message_clear_window),
				(gpointer)m);
    gnome_dialog_button_connect(GNOME_DIALOG(m->toplevel),2,
				GTK_SIGNAL_FUNC(message_destroy_event),
				(gpointer)m);
    gnome_dialog_button_connect(GNOME_DIALOG(m->toplevel),1,
				GTK_SIGNAL_FUNC(message_send),
				(gpointer)m);
    gtk_window_set_focus(GTK_WINDOW(m->toplevel),m->text_widget);  
    guiprefs_add_window(GTK_WINDOW(m->toplevel),"Server/Messages/MessageReplySize");
  } else {
    if(prefs_messages_reply_quoted==TRUE){
      m->toplevel=gnome_dialog_new(buf,_("Reply"),
				   GNOME_STOCK_BUTTON_CLOSE,NULL);
    } else {
      i=1;
      m->toplevel=gnome_dialog_new(buf,_("Reply Quoted"),_("Reply"),
				   GNOME_STOCK_BUTTON_CLOSE,NULL);
      gnome_dialog_button_connect(GNOME_DIALOG(m->toplevel),0,
				  GTK_SIGNAL_FUNC(message_open_reply_quoted),
				  (gpointer)m);
    }
    vbox=GNOME_DIALOG(m->toplevel)->vbox;
    gnome_dialog_button_connect(GNOME_DIALOG(m->toplevel),1+i,
				GTK_SIGNAL_FUNC(message_destroy_event),
				(gpointer)m);
    gnome_dialog_button_connect(GNOME_DIALOG(m->toplevel),i,
				GTK_SIGNAL_FUNC(message_open_reply),
				(gpointer)m);
    guiprefs_add_window(GTK_WINDOW(m->toplevel),"Server/Messages/MessageDialogSize");
  }
  gtk_signal_connect(GTK_OBJECT(m->toplevel),"destroy",
		     GTK_SIGNAL_FUNC(message_destroy_event),
		     (gpointer)m);
  gtk_window_set_modal(GTK_WINDOW(m->toplevel),FALSE);
  text_message=gtk_text_new(NULL,NULL);
  gtk_text_set_editable(GTK_TEXT(text_message),FALSE);
  sl=gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sl),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_AUTOMATIC);
  gtk_text_set_word_wrap(GTK_TEXT(text_message),TRUE);
  gtk_container_add(GTK_CONTAINER(sl),text_message);
  if(paned!=NULL)
    gtk_paned_pack1(GTK_PANED(paned),sl,TRUE,FALSE);
  else
    gtk_box_pack_start(GTK_BOX(vbox),sl,TRUE,TRUE,2);
  gtk_text_insert(GTK_TEXT(text_message),NULL,NULL,NULL,origmsg,-1);
  if(m->text_widget==NULL)
    m->text_widget=text_message;
  else 
    gtk_signal_connect(GTK_OBJECT(m->text_widget),"activate",
		       GTK_SIGNAL_FUNC(message_send),
		       (gpointer)m);
  return m;
}

static MessageWindow *message_handle_error(Connection *c,char *message){
  MessageWindow *m=message_create_messagewindow(c,0);
  GtkWidget *parent=main_app;
  if(c!=NULL && c->gui!=NULL && c->gui->main_window!=NULL)
    parent=c->gui->main_window;
  m->toplevel=gnome_error_dialog_parented(message,GTK_WINDOW(parent));
  play_sound(HL_SOUND_ERROR);
  return m;
}

static MessageWindow *message_handle_broadcast(Connection *c,char *message){
  MessageWindow *m=message_create_messagewindow(c,0);
  GtkWidget *parent=main_app;
  if(c!=NULL && c->gui!=NULL && c->gui->main_window!=NULL)
    parent=c->gui->main_window;
  m->toplevel=gnome_ok_dialog_parented(message,GTK_WINDOW(parent));
  play_sound(HL_SOUND_SERVMSG);
  return m;
}

void show_message_error(Connection *c,char *text){
  MessageWindow *m=message_handle_error(c,text);
  gtk_signal_connect(GTK_OBJECT(m->toplevel),"destroy",
		     GTK_SIGNAL_FUNC(message_destroy_event),
		     (gpointer)m);
  gtk_widget_show_all(m->toplevel);
}

static gboolean show_error_idle(gpointer data){
  UnkMessage *um=data;;

  if(um==NULL) return FALSE;
  if(um->c!=NULL && um->c->gui!=NULL && um->c->gui->main_window!=NULL){
    show_message_error(um->c,um->message);
  } else {
    GtkWidget *dialog,*parent;
    /* FIXME: is this neccesary? */

    parent=um->w;
    if(um->w==NULL)
      parent=main_app;
    dialog=gnome_error_dialog_parented(um->message,GTK_WINDOW(parent));
    gnome_dialog_close_hides(GNOME_DIALOG(dialog),FALSE);
    gtk_widget_show_all(dialog);
  }
  free(um->message);
  free(um);
  return FALSE;
}

void show_error(Connection *c,GtkWidget *w,char * buf){
  UnkMessage *um=malloc(sizeof(UnkMessage));
  um->c=c;
  um->w=w;
  um->message=strdup(buf);
  gtk_idle_add(show_error_idle,um);
}

void show_error_errno(Connection *c,GtkWidget *w,char *head){
  char buf[2048];

  sprintf(buf,"%s\n%s",head,strerror(errno));
  show_error(c,w,buf);
}


static gboolean handle_display_message(gpointer p){
  HLTransaction *t=(HLTransaction *)p;
  Connection *c=(Connection *)t->data;
  HLObject *message=transaction_find_object(t,HLO_MESSAGE);
  HLObject *sock=transaction_find_object(t,HLO_SOCKET);
  HLObject *nick=transaction_find_object(t,HLO_NICK);
  MessageWindow *m=NULL;

  if(message==NULL){
    transaction_destroy(t);
    return FALSE;
  }
  string_net_to_unix(message);
  if(transaction_find_object(t,HLO_PARAMETER)!=NULL)
    m=message_handle_error(c,message->data.string);
  else
    if(sock==NULL)
      m=message_handle_broadcast(c,message->data.string);
    else
      m=message_handle_privmsg(c,t,sock->data.number,
			       nick==NULL?NULL:nick->data.string,
			       message->data.string);
  if(m!=NULL && m->toplevel!=NULL){
    /*    if(c->gui!=NULL && c->gui->main_window!=NULL)
	  gnome_dialog_set_parent(GNOME_DIALOG(m->toplevel),
	  GTK_WINDOW(c->gui->main_window));
    */
    gtk_signal_connect(GTK_OBJECT(m->toplevel),"destroy",
		       GTK_SIGNAL_FUNC(message_destroy_event),
		       (gpointer)m);
    gtk_widget_show_all(m->toplevel);
  }
  transaction_destroy(t);
  return FALSE;
}

void handle_server_message(Connection *c,HLTransaction *t,gpointer data){
  transaction_read_objects(c,t);
  t->data=c;
  gtk_idle_add(handle_display_message,t);
}

