/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <gnome.h>
#include <gtk/gtk.h>
#include <gtk/gtkprogressbar.h>
#include <glib.h>

#include "tasks.h"
#include "threads.h"
#include "guiprefs.h"
#include "main.h"

static GtkWidget *task_list;
static GStaticMutex task_mutex=G_STATIC_MUTEX_INIT;
static Task *Tasks=NULL;

static void task_clicked_abort(GtkButton *but,gpointer data);

Task *task_new(const gchar *description){
  Task *t;
  GtkWidget *hbox,*vbox,*button;
  t=calloc(sizeof(Task),1);
  t->list_pos=-1;
  t->is_running=FALSE;
  t->is_download=FALSE;
  t->user_cancel=NULL;
  t->ftt=NULL;
  t->percdone=0.0;
  t->is_changed=0;
  t->widget=vbox=gtk_vbox_new(FALSE,0);
  hbox=gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);
  button=gtk_button_new_with_label(_("Abort"));
  gtk_box_pack_start(GTK_BOX(hbox),button,FALSE,FALSE,1);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(task_clicked_abort),
		     (void *)t);
  t->description=gtk_label_new(description);
  gtk_label_set_justify(GTK_LABEL(t->description),GTK_JUSTIFY_LEFT);
  gtk_box_pack_start(GTK_BOX(hbox),t->description,TRUE,TRUE,0);
  t->progressbar=gtk_progress_bar_new();
  gtk_widget_set_usize(t->progressbar,300,0);
  gtk_box_pack_end(GTK_BOX(hbox),t->progressbar,FALSE,FALSE,0);
  t->text[0]=0;
  t->label=gtk_label_new("---");
  gtk_label_set_justify(GTK_LABEL(t->label),GTK_JUSTIFY_LEFT);
  gtk_box_pack_end(GTK_BOX(vbox),t->label,FALSE,FALSE,0);
  return t;
}

static int compare_tasks(Task *a, Task *b){
  if(a->is_running!=b->is_running)
    return b->is_running-a->is_running;
  if(b->is_download!=a->is_download)
    return b->is_download-a->is_download;
  return (int)(b-a);
}

static void reorder_tasks(){
  int i=0,count;
  Task *t,**tt;

  set_cancel_task_widget_state(Tasks==NULL?FALSE:TRUE);
  if(Tasks==NULL) return;
  g_static_mutex_lock(&task_mutex);
  for(count=1,t=Tasks;t!=NULL;(count++),(t=t->next));
  tt=malloc(sizeof(Task *)*count);
  for(i=0,t=Tasks;t!=NULL;(i++),(t=t->next))
    tt[count-i-1]=t;
  qsort(tt,count,sizeof(Task *),
	(int (*)(const void *, const void *))compare_tasks);
  for(t=Tasks;t!=NULL;(t=t->next)){
    if(t->list_pos==-1){
      gtk_box_pack_start(GTK_BOX(task_list),t->widget,FALSE,FALSE,2);
      gtk_widget_show_all(t->widget);
    }
    if(t->list_pos!=i){
      t->list_pos=i;
      gtk_box_reorder_child(GTK_BOX(task_list),t->widget,t->list_pos);
    }
  }
  g_static_mutex_unlock(&task_mutex);
  free(tt);
}

static gboolean update_tasks(gpointer dummy){
  static int last_timeout=0;
  Task *t;
  gboolean need_reorder=FALSE;

  g_static_mutex_lock(&task_mutex);
  for(t=Tasks;t!=NULL;t=t->next)
    if(t->is_changed!=0){
      if(t->is_changed & TASK_CHANGED_POS){
	gtk_progress_bar_update(GTK_PROGRESS_BAR(t->progressbar),t->percdone);
	if(t->is_running==FALSE && t->percdone>0.0){
	  need_reorder=TRUE;
	  t->is_running=TRUE;
	}
      }
      if(t->is_changed & TASK_CHANGED_TEXT)
	gtk_label_set_text(GTK_LABEL(t->label),t->text);
      t->is_changed=0;
    }
  g_static_mutex_unlock(&task_mutex);
  if(need_reorder==TRUE)
    reorder_tasks();
  if(last_timeout!=prefs_task_refresh_msec){
    last_timeout=prefs_task_refresh_msec;
    g_timeout_add(prefs_task_refresh_msec,update_tasks,NULL);
    return FALSE;
  }
  return TRUE;
}

static void count_tasks(){
  Task *b;
  int i=0;
  char buf[2048];

  for(b=Tasks;b!=NULL;b=b->next)
    i++;
  sprintf(buf,_("%s (%d %s)"),APPLICATION_NAME,i,(i==1)?_("task"):_("tasks"));
  gtk_window_set_title(GTK_WINDOW(main_app),buf);
}

void task_add(Task *t){
  Task *b;

  g_static_mutex_lock(&task_mutex);
  t->next=NULL;
  if(Tasks==NULL){
    Tasks=t;
  } else 
    for(b=Tasks;b!=NULL;b=b->next){
      if(b->next==NULL){
	/* FIXME: queueing */
	b->next=t;
	break;
      }
    }
  ///  gtk_box_pack_start(GTK_BOX(task_list),t->widget,FALSE,FALSE,0);
  gtk_label_set_justify(GTK_LABEL(t->description),GTK_JUSTIFY_LEFT);
  g_static_mutex_unlock(&task_mutex);
  reorder_tasks();
  count_tasks();
}

static void nt_task_remove(Task *t){
  Task *b;
  int pos=t->list_pos;

  if(t->widget!=NULL)
    gtk_widget_destroy(t->widget);
  if(Tasks==t)
    Tasks=t->next;
  else
    for(b=Tasks;b!=NULL;b=b->next)
      if(b->next==t){
	b->next=t->next;
	break;
      }
  free(t);
  for(b=Tasks;b!=NULL;b=b->next)
    if(b->list_pos>pos)
      b->list_pos--;
  count_tasks();
}

void task_remove(Task *t){
  g_static_mutex_lock(&task_mutex);
  nt_task_remove(t);
  g_static_mutex_unlock(&task_mutex);
  reorder_tasks();
}

static gboolean task_do_deferred_remove(gpointer data){
  task_remove((Task *)data);
  return FALSE;
}

void task_remove_when_idle(Task *t){
  gtk_idle_add(task_do_deferred_remove,(gpointer)t);
}

static void user_cancel_task(Task *t){
  if(t->user_cancel!=NULL)
    t->user_cancel(t);
  nt_task_remove(t);
}

void task_update(Task *t,gboolean new_text,gfloat pos){
  if(pos>=0.0 && t->percdone!=pos){
    t->percdone=pos;
    t->is_changed|=TASK_CHANGED_POS;
  }
  if(new_text==TRUE)
    t->is_changed|=TASK_CHANGED_TEXT;
}

void remove_all_tasks(void){
  g_static_mutex_lock(&task_mutex);
  while(Tasks!=NULL)
    user_cancel_task(Tasks);
  g_static_mutex_unlock(&task_mutex);
  reorder_tasks();
}

static void task_clicked_abort(GtkButton *but,gpointer data){
  Task *t=(Task *)data;
  g_static_mutex_lock(&task_mutex);
  user_cancel_task(t);
  g_static_mutex_unlock(&task_mutex);
  reorder_tasks();
}

GtkWidget *new_task_widget(){
  if(task_list==NULL){
    gtk_timeout_add(prefs_task_refresh_msec,update_tasks,NULL);
    task_list=gtk_vbox_new(FALSE,0);
  } 
  return task_list;
}

void task_set_filetransfer(Task *t,FileTransferTask *ft){
  t->ftt=ft;
  t->is_download=TRUE;
}
