/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <glib.h>

#include "threads.h"

typedef struct {
  gpointer data;
  void (*func)(gpointer);
  void (*endfunc)(gpointer);
} NewThreadData;

static void nullfunc(gpointer dummy){}

static void stopThread(void *data){
  NewThreadData *nd=(NewThreadData *)data;
  nd->endfunc(nd->data);
  free(nd);
}

static void *startThread(void *data){
  NewThreadData *nd=(NewThreadData *)data;

  pthread_detach(pthread_self());
  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
  pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED ,NULL);
  if(nd->endfunc==NULL)
    nd->endfunc=nullfunc;
  pthread_cleanup_push(stopThread,(void *)nd);
#ifdef DEBUG
  sleep(1);
#endif
  nd->func(nd->data);
  pthread_cleanup_pop(1);
  return NULL;
}

int newThread(Thread *t,
	      void (*startFunc)(gpointer),
	      void (*endFunc)(gpointer),
	      gpointer data){
  NewThreadData *nd=malloc(sizeof(NewThreadData));
  nd->data=data;
  nd->func=startFunc;
  nd->endfunc=endFunc;

#ifdef DEBUG
  sleep(1);
#endif
  if(pthread_create(&t->pthread,NULL,
		    startThread,(void *)nd)!=0){
    free(nd);
    return FALSE;
  }
  return TRUE;
}

void killThread(Thread *t){
  pthread_cancel(t->pthread);
}

