/******************************************************************** 
   Copyright (C) 2000 Bassoukos Tassos <abas@aix.meng.auth.gr>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later
   version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnome.h>

#include "bookmarks.h"
#include "server.h"
#include "protocol.h"
#include "guiprefs.h"
#include "smalltrans.h"
#include "userlist.h"
#include "guiutils.h"
#include "hldat.h"
#include "userwidget.h"
#include "users.h"
#include "privs.h"
#include "pixmap.h"

typedef struct {
  Connection *c;
  int seluser;
  NotebookPage *notebook_page;
  UserWidget *user_list;
  UserlistOps *usersops;
  GtkWidget *user_text;
  GtkWidget *kick_dialog;
} UserNBPData;

#define USERS "UserPane"

static void users_query_user(GtkButton *b, gpointer data);
static void users_kick(GtkButton *b, gpointer data);
static void users_add_userlist(UserNBPData *u);

static void users_destroy(GtkButton *b,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  hooks_set_data(u->c,USERS,NULL);
  if(u->user_list!=NULL)
    userwidget_destroy(u->user_list);
  if(u->notebook_page!=NULL)
    gutils_nbpage_destroy(u->notebook_page);
  if(u->usersops!=NULL)
    userlist_rm_ops(u->c,u->usersops);
  if(u->kick_dialog!=NULL)
    gtk_widget_destroy(u->kick_dialog);
  u->notebook_page=NULL;
  u->user_list=NULL;
  free(u);
}

static void users_destroy_hook(Connection *c,gpointer data){
  users_destroy(NULL,data);
}

static void users_send_message(GtkButton *b,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  if(u->seluser==-1)
    return;
  message_create_simple(u->c,NULL,u->seluser);
}

static void users_refresh(GtkButton *b,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  userlist_refresh(u->c);
}

static void users_change_selecteduser(UserNBPData *u){
  gboolean f=FALSE;
  GnomeUIInfo *um;
  
  if(u->seluser!=-1)
    f=TRUE;
  gutils_nbpage_toolbar_set_sensitive(u->notebook_page,3,f);
  gutils_nbpage_toolbar_set_sensitive(u->notebook_page,4,f &&
				      privs_check(u->c,PRIV_GET_USER_INFO));
  gutils_nbpage_toolbar_set_sensitive(u->notebook_page,5,f);
  gutils_nbpage_toolbar_set_sensitive(u->notebook_page,6,f &&
				      privs_check(u->c,PRIV_DISCON_USER));
  um=userwidget_get_menu(u->user_list);
  gtk_widget_set_sensitive(um[1].widget,f&&privs_check(u->c,PRIV_GET_USER_INFO));
  gtk_widget_set_sensitive(um[4].widget,f&&privs_check(u->c,PRIV_GET_USER_INFO));
}


static void users_click_row(UserWidget *uw,int socket,
			       gboolean is_double,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  int olduser=u->seluser;

  if(socket==-1)
    u->seluser=-1;
  else 
    u->seluser=socket;
  if(u->seluser!=olduser)
    users_change_selecteduser(u);
  if(u->seluser!=-1 && is_double==TRUE)
    users_send_message(NULL,u);
}

static void users_ignore_user(GtkButton *b,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  gboolean i;
  if(u->seluser==-1)
    return;
  i=userwidget_get_disable(u->user_list,u->seluser);
  i=(i==TRUE)?FALSE:TRUE;
  userwidget_set_disable(u->user_list,u->seluser,i);
}

static void users_clear(GtkButton *b,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  gtk_editable_delete_text(GTK_EDITABLE(u->user_text),0,-1);
}

static GnomeUIInfo users_toolbar[] = {
  GNOMEUIINFO_ITEM_STOCK("Close",N_("Close userlist"),
			 users_destroy,
			 HL_STOCK_PIXMAP_CLOSE_PAGE),
  GNOMEUIINFO_ITEM_STOCK("Refresh",N_("Reload the userlist"),
			 users_refresh,
			 HL_STOCK_PIXMAP_REFRESH),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK("Message",N_("Send this user a message"),
			 users_send_message,
			 HL_STOCK_PIXMAP_USERS_SENDMESG),
  GNOMEUIINFO_ITEM_STOCK("Info",N_("Get info of a user"),
			 users_query_user,
			 HL_STOCK_PIXMAP_USERS_INFO),
  GNOMEUIINFO_ITEM_STOCK("Ignore",N_("(Un)Ignore this user"),
			 users_ignore_user,
			 GNOME_STOCK_PIXMAP_CLOSE),
  GNOMEUIINFO_ITEM_STOCK("Kick/Ban",N_("Kick or Ban this user"),
			 users_kick,
			 HL_STOCK_PIXMAP_USERS_KICK),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK("Clear",N_("Clear text window"),
			 users_clear,
			 GNOME_STOCK_PIXMAP_CLEAR),
  GNOMEUIINFO_END
};

static GnomeUIInfo users_menu[]={
  GNOMEUIINFO_ITEM_STOCK("Message",N_("Send this user a message"),
			 users_send_message,
			 HL_MENU HL_STOCK_PIXMAP_USERS_SENDMESG),
  GNOMEUIINFO_ITEM_STOCK("Info",N_("Get info of a user"),
			 users_query_user,
			 HL_MENU HL_STOCK_PIXMAP_USERS_INFO),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK("Ignore",N_("(Un)Ignore this user"),
			 users_ignore_user,
			 GNOME_STOCK_PIXMAP_CLOSE),
  GNOMEUIINFO_ITEM_STOCK("Kick/Ban",N_("Kick or Ban this user"),
			 users_kick,
			 HL_MENU HL_STOCK_PIXMAP_USERS_KICK),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_STOCK("Refresh",N_("Reload the userlist"),
			 users_refresh,
			 HL_MENU HL_STOCK_PIXMAP_REFRESH),
  GNOMEUIINFO_END
};


static void users_create_gui(UserNBPData *u){
  GtkWidget *usl;

  if(u->notebook_page!=NULL)
    return;
  u->seluser=-1;
  u->user_text=gtk_text_new(NULL,NULL);
  usl=gtk_scrolled_window_new(NULL,NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(usl),
				 GTK_POLICY_NEVER,
  				 GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(usl),u->user_text);
  u->notebook_page=gutils_nbpage_new("Main",users_toolbar,
				     u->c,_("Users"),"users",u);
  gutils_nbpage_set_main_with_userlist(u->notebook_page,usl,&u->user_list);
  userwidget_lists_userlist(u->user_list);
  userwidget_set_menu(u->user_list,users_menu,u);
  userwidget_set_select(u->user_list,users_click_row,u);
  users_change_selecteduser(u);
  users_add_userlist(u);
}

static UserNBPData *users_init(Connection *c){
  UserNBPData *u=(UserNBPData *)calloc(sizeof(UserNBPData),1);

  u->c=c;
  u->seluser=-1;
  u->notebook_page=NULL;
  u->user_list=NULL;
  u->kick_dialog=NULL;
  hooks_create(c,USERS,NULL,users_destroy_hook);
  hooks_set_data(c,USERS,(gpointer)u);
  return u;
}

static UserNBPData *get_uld(Connection *c){
  UserNBPData *u=(UserNBPData *)hooks_get_data(c,USERS);
  if(u==NULL)
    u=users_init(c);
  return u;
}

gboolean users_ignored(Connection *c,int socket){
  UserNBPData *u=get_uld(c);
  if(u==NULL) return FALSE;
  return userwidget_get_disable(u->user_list,socket);
}

void check_users(Connection *c){
  UserNBPData *u=get_uld(c);
  if(u==NULL)
    u=users_init(c);
  if(auto_open_userlist==TRUE)
    users_create_gui(u);
}

void show_users(Connection *c){
  UserNBPData *u=(UserNBPData *)hooks_get_data(c,USERS);
  if(u==NULL){
    check_users(c);
    u=(UserNBPData *)hooks_get_data(c,USERS);
  }
  if(u->notebook_page==NULL)
    users_create_gui(u);
  gutils_nbpage_to_front(u->notebook_page);
}

/* =========================== */

static void users_add(Connection *c,int sock,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  char buf[2048];
  UserlistEntry *ule=get_user_by_socket(c,sock);
  sprintf(buf,_("<< %s has connected\n"),ule->name);
  gtk_text_insert(GTK_TEXT(u->user_text),NULL,NULL,NULL,buf,-1);
}

static void users_rm(Connection *c,int sock,gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  char buf[2048];
  UserlistEntry *ule=get_user_by_socket(c,sock);
  if(ule!=NULL && ule->name!=NULL)
    sprintf(buf,_(">> %s has disconnected\n"),ule->name);
  else
    sprintf(buf,_(">> Socket %d has disconnected\n"),sock);
  gtk_text_insert(GTK_TEXT(u->user_text),NULL,NULL,NULL,buf,-1);
}

static UserlistOps uops={
  NULL,NULL,NULL,
  NULL,users_add,
  NULL,users_rm
};

static void users_add_userlist(UserNBPData *u){
  UserlistOps *up=malloc(sizeof(UserlistOps));
  *up=uops;
  u->usersops=up;
  userlist_add_ops(u->c,up,u);
}

/* =========================== */

static gboolean users_display_userinfo(gpointer data){
  HLTransaction *r=(HLTransaction *)data;
  UserNBPData *u=(UserNBPData *)r->data;
  HLObject *o=transaction_find_object(r,HLO_MESSAGE);
  
  if(o!=NULL){
    string_net_to_unix(o);
    message_display_text(u->c,o->data.string);
  }
  transaction_destroy(r);
  return FALSE;
}

static void users_user_query_reply(Connection *c,
				      HLTransaction *t,
				      HLTransaction *r,
				      gpointer data){
  UserNBPData *u=(UserNBPData *)t->data;

  if(r->w.error!=0){
    server_reply_default(c,t,r,strdup(_("Could not get user info:")));
    return;
  }
  transaction_destroy(t);
  transaction_read_objects(c,r);
  r->data=u;
  gtk_idle_add(users_display_userinfo,r);
}

static void users_send_user_query(Connection *c, gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  HLTransaction *t;
  int socket;

  if(u->seluser==-1) 
    return;
  t=transaction_new(HLCT_GETUSERINFO,NULL,FALSE);
  t->data=u;
  socket=u->seluser;
  transaction_add_object(t,create_number(HLO_SOCKET,socket));
  server_transaction_reply_set(c,t,users_user_query_reply,GINT_TO_POINTER(socket));
  transaction_send(t,c);
}

static void users_query_user(GtkButton *b, gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  
  if(u->seluser==-1) 
    return;
  server_transaction_start(u->c,users_send_user_query,u);
}

/* ================================== */

static void users_send_kickban(Connection *c,gpointer p){
  HLTransaction *t=(HLTransaction *)p;
  server_transaction_reply_set(c,t,server_reply_default,
			       strdup(_("Could not ban:")));
  transaction_send(t,c);
}

static void users_start_kickban(UserNBPData *u,gboolean is_ban){
  HLTransaction *t;

  if(u->seluser==-1)
    return;
  t=transaction_new(HLCT_KICK,u,FALSE);
  transaction_add_object(t,create_number(HLO_SOCKET,u->seluser));
  if(is_ban==TRUE)
    transaction_add_object(t,create_number(HLO_BAN,1));
  server_transaction_start(u->c,users_send_kickban,t);
}

static void users_do_kick(GtkWidget *widget,gpointer p){
  users_start_kickban((UserNBPData *)p,FALSE);
}
static void users_do_ban(GtkWidget *widget,gpointer p){
  users_start_kickban((UserNBPData *)p,TRUE);
}

static void users_abort_kick(GtkWidget *widget,gpointer p){
  UserNBPData *u=(UserNBPData *)p;
  GtkWidget *w=u->kick_dialog;
  if(w==NULL) return;
  u->kick_dialog=NULL;
  gtk_widget_destroy(w);
}
static gint users_close_kick(GtkWidget *widget,GdkEvent  *event,gpointer p){
  users_abort_kick(widget,p);
  return FALSE;
}

static void users_kick(GtkButton *b, gpointer data){
  UserNBPData *u=(UserNBPData *)data;
  UserlistEntry *ule=NULL;
  char buf[2048];

  if(u->seluser==-1) 
    return;
  ule=get_user_by_socket(u->c,u->seluser);
  if(ule==NULL)
    return;
  u->kick_dialog=gnome_dialog_new(_("Kick User?"),_("Ban"),_("Kick"),
				  GNOME_STOCK_BUTTON_CANCEL,NULL);
  gnome_dialog_set_parent(GNOME_DIALOG(u->kick_dialog),
			  GTK_WINDOW(u->c->gui->main_window));
  gnome_dialog_button_connect(GNOME_DIALOG(u->kick_dialog),2,users_abort_kick,u);
  gnome_dialog_button_connect(GNOME_DIALOG(u->kick_dialog),1,users_do_kick,u);
  gnome_dialog_button_connect(GNOME_DIALOG(u->kick_dialog),0,users_do_ban,u);
  gtk_signal_connect(GTK_OBJECT(u->kick_dialog),"delete_event",
		     GTK_SIGNAL_FUNC(users_close_kick),u);
  sprintf(buf,_("Kick user %s ?"),ule->name);
  gtk_box_pack_end(GTK_BOX(GNOME_DIALOG(u->kick_dialog)->vbox),
		   gtk_label_new(buf),FALSE,FALSE,0);
  gtk_widget_show_all(u->kick_dialog);
}
